@@ -13,6 +13,8 @@ typedef int Py_ssize_t;
 #define PY_SSIZE_T_MAX INT_MAX
 #define PY_SSIZE_T_MIN INT_MIN
 typedef Py_ssize_t (*lenfunc)(PyObject *);
+#define PyInt_FromSsize_t(x) PyInt_FromLong(x)
+typedef intargfunc ssizeargfunc;
 #endif
 
 
@@ -107,11 +109,11 @@ BaseRowProxy_init(BaseRowProxy *self, PyObject *args, PyObject *kwds)
 static PyObject *
 BaseRowProxy_reduce(PyObject *self)
 {
-	PyObject *method, *state;
-	PyObject *module, *reconstructor, *cls;
+    PyObject *method, *state;
+    PyObject *module, *reconstructor, *cls;
 
-	method = PyObject_GetAttrString(self, "__getstate__");
-	if (method == NULL)
+    method = PyObject_GetAttrString(self, "__getstate__");
+    if (method == NULL)
         return NULL;
 
     state = PyObject_CallObject(method, NULL);
@@ -239,11 +241,13 @@ static PyObject *
 BaseRowProxy_subscript(BaseRowProxy *self, PyObject *key)
 {
     PyObject *processors, *values;
-    PyObject *processor, *value;
+    PyObject *processor, *value, *processed_value;
     PyObject *row, *record, *result, *indexobject;
-    PyObject *exc_module, *exception;
+    PyObject *exc_module, *exception, *cstr_obj;
     char *cstr_key;
     long index;
+    int key_fallback = 0;
+    int tuple_check = 0;
 
     if (PyInt_CheckExact(key)) {
         index = PyInt_AS_LONG(key);
@@ -274,12 +278,17 @@ BaseRowProxy_subscript(BaseRowProxy *self, PyObject *key)
                                          "O", key);
             if (record == NULL)
                 return NULL;
+            key_fallback = 1;
         }
 
-        indexobject = PyTuple_GetItem(record, 1);
+        indexobject = PyTuple_GetItem(record, 2);
         if (indexobject == NULL)
             return NULL;
 
+        if (key_fallback) {
+            Py_DECREF(record);
+        }
+
         if (indexobject == Py_None) {
             exc_module = PyImport_ImportModule("sqlalchemy.exc");
             if (exc_module == NULL)
@@ -291,12 +300,19 @@ BaseRowProxy_subscript(BaseRowProxy *self, PyObject *key)
             if (exception == NULL)
                 return NULL;
 
-            cstr_key = PyString_AsString(key);
-            if (cstr_key == NULL)
+            // wow.  this seems quite excessive.
+            cstr_obj = PyObject_Str(key);
+            if (cstr_obj == NULL)
                 return NULL;
+            cstr_key = PyString_AsString(cstr_obj);
+            if (cstr_key == NULL) {
+                Py_DECREF(cstr_obj);
+                return NULL;
+            }
+            Py_DECREF(cstr_obj);
 
             PyErr_Format(exception,
-                    "Ambiguous column name '%s' in result set! "
+                    "Ambiguous column name '%.200s' in result set! "
                     "try 'use_labels' option on select statement.", cstr_key);
             return NULL;
         }
@@ -311,21 +327,38 @@ BaseRowProxy_subscript(BaseRowProxy *self, PyObject *key)
         return NULL;
 
     row = self->row;
-    if (PyTuple_CheckExact(row))
+    if (PyTuple_CheckExact(row)) {
         value = PyTuple_GetItem(row, index);
-    else
+        tuple_check = 1;
+    }
+    else {
         value = PySequence_GetItem(row, index);
+        tuple_check = 0;
+    }
+
     if (value == NULL)
         return NULL;
 
     if (processor != Py_None) {
-        return PyObject_CallFunctionObjArgs(processor, value, NULL);
+        processed_value = PyObject_CallFunctionObjArgs(processor, value, NULL);
+        if (!tuple_check) {
+            Py_DECREF(value);
+        }
+        return processed_value;
     } else {
-        Py_INCREF(value);
+        if (tuple_check) {
+            Py_INCREF(value);
+        }
         return value;
     }
 }
 
+static PyObject *
+BaseRowProxy_getitem(PyObject *self, Py_ssize_t i)
+{
+    return BaseRowProxy_subscript((BaseRowProxy*)self, PyInt_FromSsize_t(i));
+}
+
 static PyObject *
 BaseRowProxy_getattro(BaseRowProxy *self, PyObject *name)
 {
@@ -339,7 +372,16 @@ BaseRowProxy_getattro(BaseRowProxy *self, PyObject *name)
     else
         return tmp;
 
-    return BaseRowProxy_subscript(self, name);
+    tmp = BaseRowProxy_subscript(self, name);
+    if (tmp == NULL && PyErr_ExceptionMatches(PyExc_KeyError)) {
+        PyErr_Format(
+                PyExc_AttributeError,
+                "Could not locate column in row for column '%.200s'",
+                PyString_AsString(name)
+            );
+        return NULL;
+    }
+    return tmp;
 }
 
 /***********************
@@ -497,8 +539,8 @@ static PyGetSetDef BaseRowProxy_getseters[] = {
 static PyMethodDef BaseRowProxy_methods[] = {
     {"values", (PyCFunction)BaseRowProxy_values, METH_NOARGS,
      "Return the values represented by this BaseRowProxy as a list."},
-	{"__reduce__",  (PyCFunction)BaseRowProxy_reduce, METH_NOARGS,
-	 "Pickle support method."},
+    {"__reduce__",  (PyCFunction)BaseRowProxy_reduce, METH_NOARGS,
+     "Pickle support method."},
     {NULL}  /* Sentinel */
 };
 
@@ -506,7 +548,7 @@ static PySequenceMethods BaseRowProxy_as_sequence = {
     (lenfunc)BaseRowProxy_length,   /* sq_length */
     0,                              /* sq_concat */
     0,                              /* sq_repeat */
-    0,                              /* sq_item */
+    (ssizeargfunc)BaseRowProxy_getitem,          /* sq_item */
     0,                              /* sq_slice */
     0,                              /* sq_ass_item */
     0,                              /* sq_ass_slice */
