@@ -34,21 +34,19 @@ class CompileTest(fixtures.TestBase, AssertsExecutionResults):
 
         cls.dialect = default.DefaultDialect()
 
-    @profiling.function_call_count(versions={'2.7':62, '2.6':62,
-                                            '3':68})
+    @profiling.function_call_count(62)
     def test_insert(self):
         t1.insert().compile(dialect=self.dialect)
 
-    @profiling.function_call_count(versions={'2.6':56, '2.7':56})
+    @profiling.function_call_count(56)
     def test_update(self):
         t1.update().compile(dialect=self.dialect)
 
-    @profiling.function_call_count(versions={'2.6':117, '2.7':117, '3':118})
+    @profiling.function_call_count(110)
     def test_update_whereclause(self):
         t1.update().where(t1.c.c2==12).compile(dialect=self.dialect)
 
-    @profiling.function_call_count(versions={'2.7':148, '2.6':148,
-                                                '3':161})
+    @profiling.function_call_count(139)
     def test_select(self):
         s = select([t1], t1.c.c2==t2.c.c1)
         s.compile(dialect=self.dialect)
