@@ -1,35 +1,32 @@
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_
 from sqlalchemy.orm import mapper, relationship, create_session, \
     clear_mappers, sessionmaker, class_mapper
 from sqlalchemy.orm.mapper import _mapper_registry
 from sqlalchemy.orm.session import _sessions
-from sqlalchemy.util import jython
 import operator
-from sqlalchemy.test import testing, engines
+from test.lib import testing, engines
 from sqlalchemy import MetaData, Integer, String, ForeignKey, \
     PickleType, create_engine, Unicode
-from sqlalchemy.test.schema import Table, Column
+from test.lib.schema import Table, Column
 import sqlalchemy as sa
 from sqlalchemy.sql import column
-from sqlalchemy.test.util import gc_collect
+from sqlalchemy.processors import to_decimal_processor_factory, \
+    to_unicode_processor_factory
+from test.lib.util import gc_collect
+from sqlalchemy.util.compat import decimal
 import gc
 import weakref
-from test.orm import _base
+from test.lib import fixtures
 
-if jython:
-    from nose import SkipTest
-    raise SkipTest("Profiling not supported on this platform")
-
-
-class A(_base.ComparableEntity):
+class A(fixtures.ComparableEntity):
     pass
-class B(_base.ComparableEntity):
+class B(fixtures.ComparableEntity):
     pass
 
 def profile_memory(func):
     # run the test 50 times.  if length of gc.get_objects()
     # keeps growing, assert false
-    
+
     def profile(*args):
         gc_collect()
         samples = [0 for x in range(0, 50)]
@@ -37,11 +34,11 @@ def profile_memory(func):
             func(*args)
             gc_collect()
             samples[x] = len(gc.get_objects())
-                
+
         print "sample gc sizes:", samples
 
         assert len(_sessions) == 0
-        
+
         for x in samples[-4:]:
             if x != samples[-5]:
                 flatline = False
@@ -50,7 +47,7 @@ def profile_memory(func):
             flatline = True
 
         # object count is bigger than when it started
-        if not flatline and samples[-1] > samples[0]:  
+        if not flatline and samples[-1] > samples[0]:
             for x in samples[1:-2]:
                 # see if a spike bigger than the endpoint exists
                 if x > samples[-1]:
@@ -65,25 +62,27 @@ def assert_no_mappers():
     gc_collect()
     assert len(_mapper_registry) == 0
 
-class EnsureZeroed(_base.ORMTest):
+class EnsureZeroed(fixtures.ORMTest):
     def setup(self):
         _sessions.clear()
         _mapper_registry.clear()
 
 class MemUsageTest(EnsureZeroed):
-    
+
+    __requires__ = 'cpython',
+
     # ensure a pure growing test trips the assertion
     @testing.fails_if(lambda: True)
     def test_fixture(self):
         class Foo(object):
             pass
-            
+
         x = []
         @profile_memory
         def go():
             x[-1:] = [Foo(), Foo(), Foo(), Foo(), Foo(), Foo()]
         go()
-            
+
     def test_session(self):
         metadata = MetaData(testing.db)
 
@@ -173,10 +172,11 @@ class MemUsageTest(EnsureZeroed):
         def go():
             engine = engines.testing_engine(
                                 options={'logging_name':'FOO',
-                                        'pool_logging_name':'BAR'}
+                                        'pool_logging_name':'BAR',
+                                        'use_reaper':False}
                                     )
             sess = create_session(bind=engine)
-            
+
             a1 = A(col2="a1")
             a2 = A(col2="a2")
             a3 = A(col2="a3")
@@ -208,20 +208,49 @@ class MemUsageTest(EnsureZeroed):
         del m1, m2, m3
         assert_no_mappers()
 
+    def test_ad_hoc_types(self):
+        """test storage of bind processors, result processors
+        in dialect-wide registry."""
+
+        from sqlalchemy.dialects import mysql, postgresql, sqlite
+        from sqlalchemy import types
+
+        eng = engines.testing_engine()
+        for args in (
+            (types.Integer, ),
+            (types.String, ),
+            (types.PickleType, ),
+            (types.Enum, 'a', 'b', 'c'),
+            (sqlite.DATETIME, ),
+            (postgresql.ENUM, 'a', 'b', 'c'),
+            (types.Interval, ),
+            (postgresql.INTERVAL, ),
+            (mysql.VARCHAR, ),
+        ):
+            @profile_memory
+            def go():
+                type_ = args[0](*args[1:])
+                bp = type_._cached_bind_processor(eng.dialect)
+                rp = type_._cached_result_processor(eng.dialect, 0)
+            go()
+
+        assert not eng.dialect._type_memos
+
+
     def test_many_updates(self):
         metadata = MetaData(testing.db)
-        
+
         wide_table = Table('t', metadata,
             Column('id', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
             *[Column('col%d' % i, Integer) for i in range(10)]
         )
-        
+
         class Wide(object):
             pass
-        
+
         mapper(Wide, wide_table, _compiled_cache_size=10)
-        
+
         metadata.create_all()
         session = create_session()
         w1 = Wide()
@@ -230,7 +259,7 @@ class MemUsageTest(EnsureZeroed):
         session.close()
         del session
         counter = [1]
-        
+
         @profile_memory
         def go():
             session = create_session()
@@ -238,7 +267,7 @@ class MemUsageTest(EnsureZeroed):
             x = counter[0]
             dec = 10
             while dec > 0:
-                # trying to count in binary here, 
+                # trying to count in binary here,
                 # works enough to trip the test case
                 if pow(2, dec) < x:
                     setattr(w1, 'col%d' % dec, counter[0])
@@ -247,12 +276,12 @@ class MemUsageTest(EnsureZeroed):
             session.flush()
             session.close()
             counter[0] += 1
-            
+
         try:
             go()
         finally:
             metadata.drop_all()
-    
+
     @testing.fails_if(lambda : testing.db.dialect.name == 'sqlite' \
                       and testing.db.dialect.dbapi.version_info >= (2,
                       5),
@@ -281,7 +310,7 @@ class MemUsageTest(EnsureZeroed):
             go()
         finally:
             metadata.drop_all()
-        
+
     def test_mapper_reset(self):
         metadata = MetaData(testing.db)
 
@@ -356,7 +385,7 @@ class MemUsageTest(EnsureZeroed):
 
         @profile_memory
         def go():
-            class A(_base.ComparableEntity):
+            class A(fixtures.ComparableEntity):
                 pass
             class B(A):
                 pass
@@ -422,13 +451,13 @@ class MemUsageTest(EnsureZeroed):
 
         @profile_memory
         def go():
-            class A(_base.ComparableEntity):
+            class A(fixtures.ComparableEntity):
                 pass
-            class B(_base.ComparableEntity):
+            class B(fixtures.ComparableEntity):
                 pass
 
             mapper(A, table1, properties={
-                'bs':relationship(B, secondary=table3, 
+                'bs':relationship(B, secondary=table3,
                                     backref='as', order_by=table3.c.t1)
             })
             mapper(B, table2)
@@ -459,7 +488,7 @@ class MemUsageTest(EnsureZeroed):
             # dont need to clear_mappers()
             del B
             del A
-            
+
         metadata.create_all()
         try:
             go()
@@ -467,6 +496,22 @@ class MemUsageTest(EnsureZeroed):
             metadata.drop_all()
         assert_no_mappers()
 
+    @testing.fails_if(lambda : testing.db.dialect.name == 'sqlite' \
+                      and testing.db.dialect.dbapi.version > '2.5')
+    @testing.provide_metadata
+    def test_key_fallback_result(self):
+        e = testing.db
+        m = self.metadata
+        t = Table('t', m, Column('x', Integer), Column('y', Integer))
+        m.create_all(e)
+        e.execute(t.insert(), {"x":1, "y":1})
+        @profile_memory
+        def go():
+            r = e.execute(t.alias().select())
+            for row in r:
+                row[t.c.x]
+        go()
+
     # fails on newer versions of pysqlite due to unusual memory behvior
     # in pysqlite itself. background at:
     # http://thread.gmane.org/gmane.comp.python.db.pysqlite.user/2290
@@ -506,26 +551,26 @@ class MemUsageTest(EnsureZeroed):
             go()
         finally:
             metadata.drop_all()
-            
-            
+
+
     def test_mutable_identity(self):
         metadata = MetaData(testing.db)
 
         table1 = Table("mytable", metadata,
             Column('col1', Integer, primary_key=True,
                                 test_needs_autoincrement=True),
-            Column('col2', PickleType(comparator=operator.eq))
+            Column('col2', PickleType(comparator=operator.eq, mutable=True))
             )
-        
+
         class Foo(object):
             def __init__(self, col2):
                 self.col2 = col2
-        
+
         mapper(Foo, table1)
         metadata.create_all()
-        
+
         session = sessionmaker()()
-        
+
         def go():
             obj = [
                 Foo({'a':1}),
@@ -541,17 +586,17 @@ class MemUsageTest(EnsureZeroed):
                 Foo({'k':1}),
                 Foo({'l':1}),
             ]
-            
+
             session.add_all(obj)
             session.commit()
-            
+
             testing.eq_(len(session.identity_map._mutable_attrs), 12)
             testing.eq_(len(session.identity_map), 12)
             obj = None
             gc_collect()
             testing.eq_(len(session.identity_map._mutable_attrs), 0)
             testing.eq_(len(session.identity_map), 0)
-            
+
         try:
             go()
         finally:
@@ -565,4 +610,26 @@ class MemUsageTest(EnsureZeroed):
             dialect = SQLiteDialect()
             cast.compile(dialect=dialect)
         go()
-        
+
+    @testing.requires.cextensions
+    def test_DecimalResultProcessor_init(self):
+        @profile_memory
+        def go():
+            to_decimal_processor_factory({}, 10)
+        go()
+
+    @testing.requires.cextensions
+    def test_DecimalResultProcessor_process(self):
+        @profile_memory
+        def go():
+            to_decimal_processor_factory(decimal.Decimal, 10)(1.2)
+        go()
+
+    @testing.requires.cextensions
+    def test_UnicodeResultProcessor_init(self):
+        @profile_memory
+        def go():
+            to_unicode_processor_factory('utf8')
+        go()
+
+
