@@ -1,45 +1,47 @@
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from test.lib.testing import assert_raises, assert_raises_message
 from sqlalchemy.schema import DDL, CheckConstraint, AddConstraint, \
     DropConstraint
 from sqlalchemy import create_engine
-from sqlalchemy import MetaData, Integer, String
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from sqlalchemy import MetaData, Integer, String, event, exc, text
+from test.lib.schema import Table
+from test.lib.schema import Column
 import sqlalchemy as tsa
-from sqlalchemy.test import TestBase, testing, engines
-from sqlalchemy.test.testing import AssertsCompiledSQL
+from test.lib import testing, engines
+from test.lib.testing import AssertsCompiledSQL, eq_
 from nose import SkipTest
+from test.lib import fixtures
 
-class DDLEventTest(TestBase):
+
+class DDLEventTest(fixtures.TestBase):
     class Canary(object):
         def __init__(self, schema_item, bind):
             self.state = None
             self.schema_item = schema_item
             self.bind = bind
 
-        def before_create(self, action, schema_item, bind, **kw):
+        def before_create(self, schema_item, bind, **kw):
             assert self.state is None
             assert schema_item is self.schema_item
             assert bind is self.bind
-            self.state = action
+            self.state = 'before-create'
 
-        def after_create(self, action, schema_item, bind, **kw):
+        def after_create(self, schema_item, bind, **kw):
             assert self.state in ('before-create', 'skipped')
             assert schema_item is self.schema_item
             assert bind is self.bind
-            self.state = action
+            self.state = 'after-create'
 
-        def before_drop(self, action, schema_item, bind, **kw):
+        def before_drop(self, schema_item, bind, **kw):
             assert self.state is None
             assert schema_item is self.schema_item
             assert bind is self.bind
-            self.state = action
+            self.state = 'before-drop'
 
-        def after_drop(self, action, schema_item, bind, **kw):
+        def after_drop(self, schema_item, bind, **kw):
             assert self.state in ('before-drop', 'skipped')
             assert schema_item is self.schema_item
             assert bind is self.bind
-            self.state = action
+            self.state = 'after-drop'
 
     def setup(self):
         self.bind = engines.mock_engine()
@@ -49,7 +51,7 @@ class DDLEventTest(TestBase):
     def test_table_create_before(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['before-create'].append(canary.before_create)
+        event.listen(table, 'before_create', canary.before_create)
 
         table.create(bind)
         assert canary.state == 'before-create'
@@ -59,7 +61,7 @@ class DDLEventTest(TestBase):
     def test_table_create_after(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['after-create'].append(canary.after_create)
+        event.listen(table, 'after_create', canary.after_create)
 
         canary.state = 'skipped'
         table.create(bind)
@@ -70,8 +72,8 @@ class DDLEventTest(TestBase):
     def test_table_create_both(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['before-create'].append(canary.before_create)
-        table.ddl_listeners['after-create'].append(canary.after_create)
+        event.listen(table, 'before_create', canary.before_create)
+        event.listen(table, 'after_create', canary.after_create)
 
         table.create(bind)
         assert canary.state == 'after-create'
@@ -81,7 +83,7 @@ class DDLEventTest(TestBase):
     def test_table_drop_before(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['before-drop'].append(canary.before_drop)
+        event.listen(table, 'before_drop', canary.before_drop)
 
         table.create(bind)
         assert canary.state is None
@@ -91,7 +93,7 @@ class DDLEventTest(TestBase):
     def test_table_drop_after(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['after-drop'].append(canary.after_drop)
+        event.listen(table, 'after_drop', canary.after_drop)
 
         table.create(bind)
         assert canary.state is None
@@ -102,8 +104,9 @@ class DDLEventTest(TestBase):
     def test_table_drop_both(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['before-drop'].append(canary.before_drop)
-        table.ddl_listeners['after-drop'].append(canary.after_drop)
+
+        event.listen(table, 'before_drop', canary.before_drop)
+        event.listen(table, 'after_drop', canary.after_drop)
 
         table.create(bind)
         assert canary.state is None
@@ -113,10 +116,11 @@ class DDLEventTest(TestBase):
     def test_table_all(self):
         table, bind = self.table, self.bind
         canary = self.Canary(table, bind)
-        table.ddl_listeners['before-create'].append(canary.before_create)
-        table.ddl_listeners['after-create'].append(canary.after_create)
-        table.ddl_listeners['before-drop'].append(canary.before_drop)
-        table.ddl_listeners['after-drop'].append(canary.after_drop)
+
+        event.listen(table, 'before_create', canary.before_create)
+        event.listen(table, 'after_create', canary.after_create)
+        event.listen(table, 'before_drop', canary.before_drop)
+        event.listen(table, 'after_drop', canary.after_drop)
 
         assert canary.state is None
         table.create(bind)
@@ -128,7 +132,7 @@ class DDLEventTest(TestBase):
     def test_table_create_before(self):
         metadata, bind = self.metadata, self.bind
         canary = self.Canary(metadata, bind)
-        metadata.ddl_listeners['before-create'].append(canary.before_create)
+        event.listen(metadata, 'before_create', canary.before_create)
 
         metadata.create_all(bind)
         assert canary.state == 'before-create'
@@ -138,7 +142,7 @@ class DDLEventTest(TestBase):
     def test_metadata_create_after(self):
         metadata, bind = self.metadata, self.bind
         canary = self.Canary(metadata, bind)
-        metadata.ddl_listeners['after-create'].append(canary.after_create)
+        event.listen(metadata, 'after_create', canary.after_create)
 
         canary.state = 'skipped'
         metadata.create_all(bind)
@@ -149,8 +153,9 @@ class DDLEventTest(TestBase):
     def test_metadata_create_both(self):
         metadata, bind = self.metadata, self.bind
         canary = self.Canary(metadata, bind)
-        metadata.ddl_listeners['before-create'].append(canary.before_create)
-        metadata.ddl_listeners['after-create'].append(canary.after_create)
+
+        event.listen(metadata, 'before_create', canary.before_create)
+        event.listen(metadata, 'after_create', canary.after_create)
 
         metadata.create_all(bind)
         assert canary.state == 'after-create'
@@ -160,11 +165,11 @@ class DDLEventTest(TestBase):
     def test_metadata_table_isolation(self):
         metadata, table, bind = self.metadata, self.table, self.bind
         table_canary = self.Canary(table, bind)
-        table.ddl_listeners['before-create'
-                            ].append(table_canary.before_create)
+
+        event.listen(table, 'before_create', table_canary.before_create)
+
         metadata_canary = self.Canary(metadata, bind)
-        metadata.ddl_listeners['before-create'
-                               ].append(metadata_canary.before_create)
+        event.listen(metadata, 'before_create', metadata_canary.before_create)
         self.table.create(self.bind)
         assert metadata_canary.state == None
 
@@ -174,13 +179,15 @@ class DDLEventTest(TestBase):
         fn = lambda *a: None
 
         table.append_ddl_listener('before-create', fn)
-        assert_raises(LookupError, table.append_ddl_listener, 'blah', fn)
+        assert_raises(exc.InvalidRequestError, table.append_ddl_listener,
+                                        'blah', fn)
 
         metadata.append_ddl_listener('before-create', fn)
-        assert_raises(LookupError, metadata.append_ddl_listener, 'blah', fn)
+        assert_raises(exc.InvalidRequestError, metadata.append_ddl_listener,
+                                        'blah', fn)
 
 
-class DDLExecutionTest(TestBase):
+class DDLExecutionTest(fixtures.TestBase):
     def setup(self):
         self.engine = engines.mock_engine()
         self.metadata = MetaData(self.engine)
@@ -191,10 +198,10 @@ class DDLExecutionTest(TestBase):
 
     def test_table_standalone(self):
         users, engine = self.users, self.engine
-        DDL('mxyzptlk').execute_at('before-create', users)
-        DDL('klptzyxm').execute_at('after-create', users)
-        DDL('xyzzy').execute_at('before-drop', users)
-        DDL('fnord').execute_at('after-drop', users)
+        event.listen(users, 'before_create', DDL('mxyzptlk'))
+        event.listen(users, 'after_create', DDL('klptzyxm'))
+        event.listen(users, 'before_drop', DDL('xyzzy'))
+        event.listen(users, 'after_drop', DDL('fnord'))
 
         users.create()
         strings = [str(x) for x in engine.mock]
@@ -212,6 +219,29 @@ class DDLExecutionTest(TestBase):
 
     def test_table_by_metadata(self):
         metadata, users, engine = self.metadata, self.users, self.engine
+
+        event.listen(users, 'before_create', DDL('mxyzptlk'))
+        event.listen(users, 'after_create', DDL('klptzyxm'))
+        event.listen(users, 'before_drop', DDL('xyzzy'))
+        event.listen(users, 'after_drop', DDL('fnord'))
+
+        metadata.create_all()
+        strings = [str(x) for x in engine.mock]
+        assert 'mxyzptlk' in strings
+        assert 'klptzyxm' in strings
+        assert 'xyzzy' not in strings
+        assert 'fnord' not in strings
+        del engine.mock[:]
+        metadata.drop_all()
+        strings = [str(x) for x in engine.mock]
+        assert 'mxyzptlk' not in strings
+        assert 'klptzyxm' not in strings
+        assert 'xyzzy' in strings
+        assert 'fnord' in strings
+
+    @testing.uses_deprecated(r'See DDLEvents')
+    def test_table_by_metadata_deprecated(self):
+        metadata, users, engine = self.metadata, self.users, self.engine
         DDL('mxyzptlk').execute_at('before-create', users)
         DDL('klptzyxm').execute_at('after-create', users)
         DDL('xyzzy').execute_at('before-drop', users)
@@ -230,7 +260,110 @@ class DDLExecutionTest(TestBase):
         assert 'klptzyxm' not in strings
         assert 'xyzzy' in strings
         assert 'fnord' in strings
-    
+
+    def test_deprecated_append_ddl_listener_table(self):
+        metadata, users, engine = self.metadata, self.users, self.engine
+        canary = []
+        users.append_ddl_listener('before-create',
+                            lambda e, t, b:canary.append('mxyzptlk')
+                        )
+        users.append_ddl_listener('after-create',
+                            lambda e, t, b:canary.append('klptzyxm')
+                        )
+        users.append_ddl_listener('before-drop',
+                            lambda e, t, b:canary.append('xyzzy')
+                        )
+        users.append_ddl_listener('after-drop',
+                            lambda e, t, b:canary.append('fnord')
+                        )
+
+        metadata.create_all()
+        assert 'mxyzptlk' in canary
+        assert 'klptzyxm' in canary
+        assert 'xyzzy' not in canary
+        assert 'fnord' not in canary
+        del engine.mock[:]
+        canary[:] = []
+        metadata.drop_all()
+        assert 'mxyzptlk' not in canary
+        assert 'klptzyxm' not in canary
+        assert 'xyzzy' in canary
+        assert 'fnord' in canary
+
+    def test_deprecated_append_ddl_listener_metadata(self):
+        metadata, users, engine = self.metadata, self.users, self.engine
+        canary = []
+        metadata.append_ddl_listener('before-create',
+                            lambda e, t, b, tables=None:canary.append('mxyzptlk')
+                        )
+        metadata.append_ddl_listener('after-create',
+                            lambda e, t, b, tables=None:canary.append('klptzyxm')
+                        )
+        metadata.append_ddl_listener('before-drop',
+                            lambda e, t, b, tables=None:canary.append('xyzzy')
+                        )
+        metadata.append_ddl_listener('after-drop',
+                            lambda e, t, b, tables=None:canary.append('fnord')
+                        )
+
+        metadata.create_all()
+        assert 'mxyzptlk' in canary
+        assert 'klptzyxm' in canary
+        assert 'xyzzy' not in canary
+        assert 'fnord' not in canary
+        del engine.mock[:]
+        canary[:] = []
+        metadata.drop_all()
+        assert 'mxyzptlk' not in canary
+        assert 'klptzyxm' not in canary
+        assert 'xyzzy' in canary
+        assert 'fnord' in canary
+
+    def test_metadata(self):
+        metadata, engine = self.metadata, self.engine
+
+        event.listen(metadata, 'before_create', DDL('mxyzptlk'))
+        event.listen(metadata, 'after_create', DDL('klptzyxm'))
+        event.listen(metadata, 'before_drop', DDL('xyzzy'))
+        event.listen(metadata, 'after_drop', DDL('fnord'))
+
+        metadata.create_all()
+        strings = [str(x) for x in engine.mock]
+        assert 'mxyzptlk' in strings
+        assert 'klptzyxm' in strings
+        assert 'xyzzy' not in strings
+        assert 'fnord' not in strings
+        del engine.mock[:]
+        metadata.drop_all()
+        strings = [str(x) for x in engine.mock]
+        assert 'mxyzptlk' not in strings
+        assert 'klptzyxm' not in strings
+        assert 'xyzzy' in strings
+        assert 'fnord' in strings
+
+    @testing.uses_deprecated(r'See DDLEvents')
+    def test_metadata_deprecated(self):
+        metadata, engine = self.metadata, self.engine
+
+        DDL('mxyzptlk').execute_at('before-create', metadata)
+        DDL('klptzyxm').execute_at('after-create', metadata)
+        DDL('xyzzy').execute_at('before-drop', metadata)
+        DDL('fnord').execute_at('after-drop', metadata)
+
+        metadata.create_all()
+        strings = [str(x) for x in engine.mock]
+        assert 'mxyzptlk' in strings
+        assert 'klptzyxm' in strings
+        assert 'xyzzy' not in strings
+        assert 'fnord' not in strings
+        del engine.mock[:]
+        metadata.drop_all()
+        strings = [str(x) for x in engine.mock]
+        assert 'mxyzptlk' not in strings
+        assert 'klptzyxm' not in strings
+        assert 'xyzzy' in strings
+        assert 'fnord' in strings
+
     def test_conditional_constraint(self):
         metadata, users, engine = self.metadata, self.users, self.engine
         nonpg_mock = engines.mock_engine(dialect_name='sqlite')
@@ -241,6 +374,42 @@ class DDLExecutionTest(TestBase):
         # by placing the constraint in an Add/Drop construct, the
         # 'inline_ddl' flag is set to False
 
+        event.listen(
+            users,
+            'after_create',
+            AddConstraint(constraint).execute_if(dialect='postgresql'),
+        )
+
+        event.listen(
+            users,
+            'before_drop',
+            DropConstraint(constraint).execute_if(dialect='postgresql'),
+        )
+
+        metadata.create_all(bind=nonpg_mock)
+        strings = ' '.join(str(x) for x in nonpg_mock.mock)
+        assert 'my_test_constraint' not in strings
+        metadata.drop_all(bind=nonpg_mock)
+        strings = ' '.join(str(x) for x in nonpg_mock.mock)
+        assert 'my_test_constraint' not in strings
+        metadata.create_all(bind=pg_mock)
+        strings = ' '.join(str(x) for x in pg_mock.mock)
+        assert 'my_test_constraint' in strings
+        metadata.drop_all(bind=pg_mock)
+        strings = ' '.join(str(x) for x in pg_mock.mock)
+        assert 'my_test_constraint' in strings
+
+    @testing.uses_deprecated(r'See DDLEvents')
+    def test_conditional_constraint_deprecated(self):
+        metadata, users, engine = self.metadata, self.users, self.engine
+        nonpg_mock = engines.mock_engine(dialect_name='sqlite')
+        pg_mock = engines.mock_engine(dialect_name='postgresql')
+        constraint = CheckConstraint('a < b', name='my_test_constraint'
+                , table=users)
+
+        # by placing the constraint in an Add/Drop construct, the
+        # 'inline_ddl' flag is set to False
+
         AddConstraint(constraint, on='postgresql'
                       ).execute_at('after-create', users)
         DropConstraint(constraint, on='postgresql'
@@ -257,27 +426,6 @@ class DDLExecutionTest(TestBase):
         metadata.drop_all(bind=pg_mock)
         strings = ' '.join(str(x) for x in pg_mock.mock)
         assert 'my_test_constraint' in strings
-        
-    def test_metadata(self):
-        metadata, engine = self.metadata, self.engine
-        DDL('mxyzptlk').execute_at('before-create', metadata)
-        DDL('klptzyxm').execute_at('after-create', metadata)
-        DDL('xyzzy').execute_at('before-drop', metadata)
-        DDL('fnord').execute_at('after-drop', metadata)
-
-        metadata.create_all()
-        strings = [str(x) for x in engine.mock]
-        assert 'mxyzptlk' in strings
-        assert 'klptzyxm' in strings
-        assert 'xyzzy' not in strings
-        assert 'fnord' not in strings
-        del engine.mock[:]
-        metadata.drop_all()
-        strings = [str(x) for x in engine.mock]
-        assert 'mxyzptlk' not in strings
-        assert 'klptzyxm' not in strings
-        assert 'xyzzy' in strings
-        assert 'fnord' in strings
 
     def test_ddl_execute(self):
         try:
@@ -314,7 +462,31 @@ class DDLExecutionTest(TestBase):
                 r = eval(py)
                 assert list(r) == [(1,)], py
 
-class DDLTest(TestBase, AssertsCompiledSQL):
+    @testing.fails_on('postgresql+pg8000', 'pg8000 requires explicit types')
+    def test_platform_escape(self):
+        """test the escaping of % characters in the DDL construct."""
+
+        default_from = testing.db.dialect.statement_compiler(
+                            testing.db.dialect, None).default_from()
+
+        eq_(
+            testing.db.execute(
+                text("select 'foo%something'" + default_from)
+            ).scalar(),
+            'foo%something'
+        )
+
+        eq_(
+            testing.db.execute(
+                DDL("select 'foo%%something'" + default_from)
+            ).scalar(),
+            'foo%something'
+        )
+
+
+
+
+class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
     def mock_engine(self):
         executor = lambda *a, **kw: None
         engine = create_engine(testing.db.name + '://',
@@ -355,18 +527,39 @@ class DDLTest(TestBase, AssertsCompiledSQL):
         self.assert_compile(ddl.against(insane_schema),
                             'S S-T T-"s s"."t t"-b', dialect=dialect)
 
+
     def test_filter(self):
         cx = self.mock_engine()
 
         tbl = Table('t', MetaData(), Column('id', Integer))
         target = cx.name
 
-        assert DDL('')._should_execute('x', tbl, cx)
-        assert DDL('', on=target)._should_execute('x', tbl, cx)
-        assert not DDL('', on='bogus')._should_execute('x', tbl, cx)
-        assert DDL('', on=lambda d, x,y,z: True)._should_execute('x', tbl, cx)
+        assert DDL('')._should_execute(tbl, cx)
+        assert DDL('').execute_if(dialect=target)._should_execute(tbl, cx)
+        assert not DDL('').execute_if(dialect='bogus').\
+                        _should_execute(tbl, cx)
+        assert DDL('').execute_if(callable_=lambda d, y,z, **kw: True).\
+                        _should_execute(tbl, cx)
+        assert(DDL('').execute_if(
+                        callable_=lambda d, y,z, **kw: z.engine.name
+                        != 'bogus').
+               _should_execute(tbl, cx))
+
+    @testing.uses_deprecated(r'See DDLEvents')
+    def test_filter_deprecated(self):
+        cx = self.mock_engine()
+
+        tbl = Table('t', MetaData(), Column('id', Integer))
+        target = cx.name
+
+        assert DDL('')._should_execute_deprecated('x', tbl, cx)
+        assert DDL('', on=target)._should_execute_deprecated('x', tbl, cx)
+        assert not DDL('', on='bogus').\
+                        _should_execute_deprecated('x', tbl, cx)
+        assert DDL('', on=lambda d, x,y,z: True).\
+                        _should_execute_deprecated('x', tbl, cx)
         assert(DDL('', on=lambda d, x,y,z: z.engine.name != 'bogus').
-               _should_execute('x', tbl, cx))
+               _should_execute_deprecated('x', tbl, cx))
 
     def test_repr(self):
         assert repr(DDL('s'))
