@@ -1,14 +1,13 @@
-from sqlalchemy.test.testing import assert_raises, assert_raises_message, eq_
+from test.lib.testing import assert_raises, assert_raises_message, eq_
 import ConfigParser
 import StringIO
 import sqlalchemy.engine.url as url
-from sqlalchemy import create_engine, engine_from_config
+from sqlalchemy import create_engine, engine_from_config, exc, pool
 from sqlalchemy.engine import _coerce_config
 import sqlalchemy as tsa
-from sqlalchemy.test import TestBase
+from test.lib import fixtures, testing
 
-
-class ParseConnectTest(TestBase):
+class ParseConnectTest(fixtures.TestBase):
     def test_rfc1738(self):
         for text in (
             'dbtype://username:password@hostspec:110//usr/db_file.db',
@@ -40,7 +39,7 @@ class ParseConnectTest(TestBase):
                 or not u.host
             assert str(u) == text
 
-class DialectImportTest(TestBase):
+class DialectImportTest(fixtures.TestBase):
     def test_import_base_dialects(self):
 
         # the globals() somehow makes it for the exec() + nose3.
@@ -57,7 +56,7 @@ class DialectImportTest(TestBase):
                   '%s.dialect()' % (name, name), globals())
             eq_(dialect.name, name)
 
-class CreateEngineTest(TestBase):
+class CreateEngineTest(fixtures.TestBase):
     """test that create_engine arguments of different types get
     propagated properly"""
 
@@ -140,7 +139,7 @@ pool_timeout=10
         assert e.echo is True
 
         for param, values in [
-            ('convert_unicode', ('true', 'false', 'force')), 
+            ('convert_unicode', ('true', 'false', 'force')),
             ('echo', ('true', 'false', 'debug')),
             ('echo_pool', ('true', 'false', 'debug')),
             ('use_native_unicode', ('true', 'false')),
@@ -176,8 +175,29 @@ pool_timeout=10
                           module=dbapi, _initialize=False)
         assert e.pool._recycle == 472
 
-    def test_badargs(self):
-        assert_raises(ImportError, create_engine, 'foobar://',
+    def test_reset_on_return(self):
+        dbapi = MockDBAPI(foober=12, lala=18, hoho={'this': 'dict'},
+                          fooz='somevalue')
+        for (value, expected) in [
+            ('rollback', pool.reset_rollback),
+            ('commit', pool.reset_commit),
+            (None, pool.reset_none),
+            (True, pool.reset_rollback),
+            (False, pool.reset_none),
+        ]:
+            e = create_engine('postgresql://', pool_reset_on_return=value,
+                          module=dbapi, _initialize=False)
+            assert e.pool._reset_on_return is expected
+
+        assert_raises(
+            exc.ArgumentError,
+            create_engine, "postgresql://",
+            pool_reset_on_return='hi', module=dbapi,
+            _initialize=False
+        )
+
+    def test_bad_args(self):
+        assert_raises(exc.ArgumentError, create_engine, 'foobar://',
                       module=mock_dbapi)
 
         # bad arg
@@ -202,19 +222,50 @@ pool_timeout=10
         assert_raises(TypeError, create_engine, 'mysql+mysqldb://',
                       use_unicode=True, module=mock_dbapi)
 
+    @testing.requires.sqlite
+    def test_wraps_connect_in_dbapi(self):
         # sqlite uses SingletonThreadPool which doesnt have max_overflow
 
         assert_raises(TypeError, create_engine, 'sqlite://',
                       max_overflow=5, module=mock_sqlite_dbapi)
+        e = create_engine('sqlite://', connect_args={'use_unicode'
+                          : True}, convert_unicode=True)
         try:
-            e = create_engine('sqlite://', connect_args={'use_unicode'
-                              : True}, convert_unicode=True)
-        except ImportError:
-            # no sqlite
-            pass
-        else:
-            # raises DBAPIerror due to use_unicode not a sqlite arg
-            assert_raises(tsa.exc.DBAPIError, e.connect)
+            e.connect()
+        except tsa.exc.DBAPIError, de:
+            assert not de.connection_invalidated
+
+    def test_ensure_dialect_does_is_disconnect_no_conn(self):
+        """test that is_disconnect() doesn't choke if no connection, cursor given."""
+        dialect = testing.db.dialect
+        dbapi = dialect.dbapi
+        assert not dialect.is_disconnect(dbapi.OperationalError("test"), None, None)
+
+    @testing.requires.sqlite
+    def test_invalidate_on_connect(self):
+        """test that is_disconnect() is called during connect.
+
+        interpretation of connection failures are not supported by
+        every backend.
+
+        """
+        # pretend pysqlite throws the
+        # "Cannot operate on a closed database." error
+        # on connect.   IRL we'd be getting Oracle's "shutdown in progress"
+
+        e = create_engine('sqlite://')
+        sqlite3 = e.dialect.dbapi
+        class ThrowOnConnect(MockDBAPI):
+            dbapi = sqlite3
+            Error = sqlite3.Error
+            ProgrammingError = sqlite3.ProgrammingError
+            def connect(self, *args, **kw):
+                raise sqlite3.ProgrammingError("Cannot operate on a closed database.")
+        try:
+            create_engine('sqlite://', module=ThrowOnConnect()).connect()
+            assert False
+        except tsa.exc.DBAPIError, de:
+            assert de.connection_invalidated
 
     def test_urlattr(self):
         """test the url attribute on ``Engine``."""
@@ -265,6 +316,9 @@ pool_timeout=10
             )
 
 class MockDBAPI(object):
+    version_info = sqlite_version_info = 99, 9, 9
+    sqlite_version = '99.9.9'
+
     def __init__(self, **kwargs):
         self.kwargs = kwargs
         self.paramstyle = 'named'
@@ -294,5 +348,3 @@ class MockCursor(object):
 
 mock_dbapi = MockDBAPI()
 mock_sqlite_dbapi = msd = MockDBAPI()
-msd.version_info = msd.sqlite_version_info = 99, 9, 9
-msd.sqlite_version = '99.9.9'
