@@ -1,14 +1,15 @@
 import threading, time
-from sqlalchemy import pool, interfaces, create_engine, select
+from sqlalchemy import pool, interfaces, select, event
 import sqlalchemy as tsa
-from sqlalchemy.test import TestBase, testing
-from sqlalchemy.test.util import gc_collect, lazy_gc
-from sqlalchemy.test.testing import eq_
+from test.lib import testing
+from test.lib.util import gc_collect, lazy_gc
+from test.lib.testing import eq_, assert_raises
+from test.lib.engines import testing_engine
+from test.lib import fixtures
 
 mcid = 1
 class MockDBAPI(object):
-    def __init__(self):
-        self.throw_error = False
+    throw_error = False
     def connect(self, *args, **kwargs):
         if self.throw_error:
             raise Exception("couldnt connect !")
@@ -17,10 +18,10 @@ class MockDBAPI(object):
             time.sleep(delay)
         return MockConnection()
 class MockConnection(object):
+    closed = False
     def __init__(self):
         global mcid
         self.id = mcid
-        self.closed = False
         mcid += 1
     def close(self):
         self.closed = True
@@ -33,10 +34,8 @@ class MockCursor(object):
         pass
     def close(self):
         pass
-mock_dbapi = MockDBAPI()
 
-
-class PoolTestBase(TestBase):
+class PoolTestBase(fixtures.TestBase):
     def setup(self):
         pool.clear_managers()
 
@@ -44,28 +43,52 @@ class PoolTestBase(TestBase):
     def teardown_class(cls):
        pool.clear_managers()
 
+    def _queuepool_fixture(self, **kw):
+        dbapi = MockDBAPI()
+        return pool.QueuePool(creator=lambda: dbapi.connect('foo.db'), **kw)
+
+    def _queuepool_dbapi_fixture(self, **kw):
+        dbapi = MockDBAPI()
+        return dbapi, pool.QueuePool(creator=lambda: dbapi.connect('foo.db'), **kw)
+
 class PoolTest(PoolTestBase):
-    def testmanager(self):
-        manager = pool.manage(mock_dbapi, use_threadlocal=True)
+    def test_manager(self):
+        manager = pool.manage(MockDBAPI(), use_threadlocal=True)
 
-        connection = manager.connect('foo.db')
-        connection2 = manager.connect('foo.db')
-        connection3 = manager.connect('bar.db')
+        c1 = manager.connect('foo.db')
+        c2 = manager.connect('foo.db')
+        c3 = manager.connect('bar.db')
+        c4 = manager.connect("foo.db", bar="bat")
+        c5 = manager.connect("foo.db", bar="hoho")
+        c6 = manager.connect("foo.db", bar="bat")
 
-        self.assert_(connection.cursor() is not None)
-        self.assert_(connection is connection2)
-        self.assert_(connection2 is not connection3)
+        assert c1.cursor() is not None
+        assert c1 is c2
+        assert c1 is not c3
+        assert c4 is c6
+        assert c4 is not c5
 
-    def testbadargs(self):
-        manager = pool.manage(mock_dbapi)
+    def test_manager_with_key(self):
+        class NoKws(object):
+            def connect(self, arg):
+                return MockConnection()
 
-        try:
-            connection = manager.connect(None)
-        except:
-            pass
+        manager = pool.manage(NoKws(), use_threadlocal=True)
 
-    def testnonthreadlocalmanager(self):
-        manager = pool.manage(mock_dbapi, use_threadlocal = False)
+        c1 = manager.connect('foo.db', sa_pool_key="a")
+        c2 = manager.connect('foo.db', sa_pool_key="b")
+        c3 = manager.connect('bar.db', sa_pool_key="a")
+
+        assert c1.cursor() is not None
+        assert c1 is not c2
+        assert c1 is  c3
+
+    def test_bad_args(self):
+        manager = pool.manage(MockDBAPI())
+        connection = manager.connect(None)
+
+    def test_non_thread_local_manager(self):
+        manager = pool.manage(MockDBAPI(), use_threadlocal = False)
 
         connection = manager.connect('foo.db')
         connection2 = manager.connect('foo.db')
@@ -87,11 +110,12 @@ class PoolTest(PoolTestBase):
         def creator():
             raise Exception("no creates allowed")
 
-        for cls in (pool.SingletonThreadPool, pool.StaticPool, 
+        for cls in (pool.SingletonThreadPool, pool.StaticPool,
                     pool.QueuePool, pool.NullPool, pool.AssertionPool):
             p = cls(creator=creator)
             p.dispose()
-            p.recreate()
+            p2 = p.recreate()
+            assert p2.__class__ is cls
 
             mock_dbapi = MockDBAPI()
             p = cls(creator=mock_dbapi.connect)
@@ -101,7 +125,6 @@ class PoolTest(PoolTestBase):
             p.dispose()
             p.recreate()
 
-
     def testthreadlocal_del(self):
         self._do_testthreadlocal(useclose=False)
 
@@ -109,10 +132,11 @@ class PoolTest(PoolTestBase):
         self._do_testthreadlocal(useclose=True)
 
     def _do_testthreadlocal(self, useclose=False):
-        for p in pool.QueuePool(creator=mock_dbapi.connect,
+        dbapi = MockDBAPI()
+        for p in pool.QueuePool(creator=dbapi.connect,
                                 pool_size=3, max_overflow=-1,
                                 use_threadlocal=True), \
-            pool.SingletonThreadPool(creator=mock_dbapi.connect,
+            pool.SingletonThreadPool(creator=dbapi.connect,
                 use_threadlocal=True):
             c1 = p.connect()
             c2 = p.connect()
@@ -159,9 +183,7 @@ class PoolTest(PoolTestBase):
                 self.assert_(p.checkedout() == 0)
 
     def test_properties(self):
-        dbapi = MockDBAPI()
-        p = pool.QueuePool(creator=lambda: dbapi.connect('foo.db'),
-                           pool_size=1, max_overflow=0, use_threadlocal=False)
+        p = self._queuepool_fixture(pool_size=1, max_overflow=0)
 
         c = p.connect()
         self.assert_(not c.info)
@@ -187,9 +209,225 @@ class PoolTest(PoolTestBase):
         self.assert_(not c2.info)
         self.assert_('foo2' in c.info)
 
-    def test_listeners(self):
-        dbapi = MockDBAPI()
 
+
+class PoolEventsTest(object): #PoolTestBase):
+    def _first_connect_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = []
+        def first_connect(*arg, **kw):
+            canary.append('first_connect')
+
+        event.listen(p, 'first_connect', first_connect)
+
+        return p, canary
+
+    def _connect_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = []
+        def connect(*arg, **kw):
+            canary.append('connect')
+        event.listen(p, 'connect', connect)
+
+        return p, canary
+
+    def _checkout_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = []
+        def checkout(*arg, **kw):
+            canary.append('checkout')
+        event.listen(p, 'checkout', checkout)
+
+        return p, canary
+
+    def _checkin_event_fixture(self):
+        p = self._queuepool_fixture()
+        canary = []
+        def checkin(*arg, **kw):
+            canary.append('checkin')
+        event.listen(p, 'checkin', checkin)
+
+        return p, canary
+
+    def test_first_connect_event(self):
+        p, canary = self._first_connect_event_fixture()
+
+        c1 = p.connect()
+        eq_(canary, ['first_connect'])
+
+    def test_first_connect_event_fires_once(self):
+        p, canary = self._first_connect_event_fixture()
+
+        c1 = p.connect()
+        c2 = p.connect()
+
+        eq_(canary, ['first_connect'])
+
+    def test_first_connect_on_previously_recreated(self):
+        p, canary = self._first_connect_event_fixture()
+
+        p2 = p.recreate()
+        c1 = p.connect()
+        c2 = p2.connect()
+
+        eq_(canary, ['first_connect', 'first_connect'])
+
+    def test_first_connect_on_subsequently_recreated(self):
+        p, canary = self._first_connect_event_fixture()
+
+        c1 = p.connect()
+        p2 = p.recreate()
+        c2 = p2.connect()
+
+        eq_(canary, ['first_connect', 'first_connect'])
+
+    def test_connect_event(self):
+        p, canary = self._connect_event_fixture()
+
+        c1 = p.connect()
+        eq_(canary, ['connect'])
+
+    def test_connect_event_fires_subsequent(self):
+        p, canary = self._connect_event_fixture()
+
+        c1 = p.connect()
+        c2 = p.connect()
+
+        eq_(canary, ['connect', 'connect'])
+
+    def test_connect_on_previously_recreated(self):
+        p, canary = self._connect_event_fixture()
+
+        p2 = p.recreate()
+
+        c1 = p.connect()
+        c2 = p2.connect()
+
+        eq_(canary, ['connect', 'connect'])
+
+    def test_connect_on_subsequently_recreated(self):
+        p, canary = self._connect_event_fixture()
+
+        c1 = p.connect()
+        p2 = p.recreate()
+        c2 = p2.connect()
+
+        eq_(canary, ['connect', 'connect'])
+
+    def test_checkout_event(self):
+        p, canary = self._checkout_event_fixture()
+
+        c1 = p.connect()
+        eq_(canary, ['checkout'])
+
+    def test_checkout_event_fires_subsequent(self):
+        p, canary = self._checkout_event_fixture()
+
+        c1 = p.connect()
+        c2 = p.connect()
+        eq_(canary, ['checkout', 'checkout'])
+
+    def test_checkout_event_on_subsequently_recreated(self):
+        p, canary = self._checkout_event_fixture()
+
+        c1 = p.connect()
+        p2 = p.recreate()
+        c2 = p2.connect()
+
+        eq_(canary, ['checkout', 'checkout'])
+
+    def test_checkin_event(self):
+        p, canary = self._checkin_event_fixture()
+
+        c1 = p.connect()
+        eq_(canary, [])
+        c1.close()
+        eq_(canary, ['checkin'])
+
+    def test_checkin_event_gc(self):
+        p, canary = self._checkin_event_fixture()
+
+        c1 = p.connect()
+        eq_(canary, [])
+        del c1
+        lazy_gc()
+        eq_(canary, ['checkin'])
+
+    def test_checkin_event_on_subsequently_recreated(self):
+        p, canary = self._checkin_event_fixture()
+
+        c1 = p.connect()
+        p2 = p.recreate()
+        c2 = p2.connect()
+
+        eq_(canary, [])
+
+        c1.close()
+        eq_(canary, ['checkin'])
+
+        c2.close()
+        eq_(canary, ['checkin', 'checkin'])
+
+    def test_listen_targets_scope(self):
+        canary = []
+        def listen_one(*args):
+            canary.append("listen_one")
+        def listen_two(*args):
+            canary.append("listen_two")
+        def listen_three(*args):
+            canary.append("listen_three")
+        def listen_four(*args):
+            canary.append("listen_four")
+
+        engine = testing_engine(testing.db.url)
+        event.listen(pool.Pool, 'connect', listen_one)
+        event.listen(engine.pool, 'connect', listen_two)
+        event.listen(engine, 'connect', listen_three)
+        event.listen(engine.__class__, 'connect', listen_four)
+
+        engine.execute(select([1])).close()
+        eq_(
+            canary, ["listen_one","listen_four", "listen_two","listen_three"]
+        )
+
+    def test_listen_targets_per_subclass(self):
+        """test that listen() called on a subclass remains specific to that subclass."""
+
+        canary = []
+        def listen_one(*args):
+            canary.append("listen_one")
+        def listen_two(*args):
+            canary.append("listen_two")
+        def listen_three(*args):
+            canary.append("listen_three")
+
+        event.listen(pool.Pool, 'connect', listen_one)
+        event.listen(pool.QueuePool, 'connect', listen_two)
+        event.listen(pool.SingletonThreadPool, 'connect', listen_three)
+
+        p1 = pool.QueuePool(creator=MockDBAPI().connect)
+        p2 = pool.SingletonThreadPool(creator=MockDBAPI().connect)
+
+        assert listen_one in p1.dispatch.connect
+        assert listen_two in p1.dispatch.connect
+        assert listen_three not in p1.dispatch.connect
+        assert listen_one in p2.dispatch.connect
+        assert listen_two not in p2.dispatch.connect
+        assert listen_three in p2.dispatch.connect
+
+        p1.connect()
+        eq_(canary, ["listen_one", "listen_two"])
+        p2.connect()
+        eq_(canary, ["listen_one", "listen_two", "listen_one", "listen_three"])
+
+    def teardown(self):
+        # TODO: need to get remove() functionality
+        # going
+        pool.Pool.dispatch._clear()
+
+class DeprecatedPoolListenerTest(PoolTestBase):
+    @testing.uses_deprecated(r".*Use event.listen")
+    def test_listeners(self):
         class InstrumentingListener(object):
             def __init__(self):
                 if hasattr(self, 'connect'):
@@ -211,7 +449,7 @@ class PoolTest(PoolTestBase):
                 eq_(len(innerself.first_connected), fconn)
                 eq_(len(innerself.checked_out), cout)
                 eq_(len(innerself.checked_in), cin)
-            def assert_in(innerself, item, in_conn, in_fconn, 
+            def assert_in(innerself, item, in_conn, in_fconn,
                                                 in_cout, in_cin):
                 self.assert_((item in innerself.connected) == in_conn)
                 self.assert_((item in innerself.first_connected) == in_fconn)
@@ -254,19 +492,14 @@ class PoolTest(PoolTestBase):
             def checkin(self, con, record):
                 pass
 
-        def _pool(**kw):
-            return pool.QueuePool(creator=lambda: dbapi.connect('foo.db'),
-                                  use_threadlocal=False, **kw)
-
         def assert_listeners(p, total, conn, fconn, cout, cin):
             for instance in (p, p.recreate()):
-                self.assert_(len(instance.listeners) == total)
-                self.assert_(len(instance._on_connect) == conn)
-                self.assert_(len(instance._on_first_connect) == fconn)
-                self.assert_(len(instance._on_checkout) == cout)
-                self.assert_(len(instance._on_checkin) == cin)
+                self.assert_(len(instance.dispatch.connect) == conn)
+                self.assert_(len(instance.dispatch.first_connect) == fconn)
+                self.assert_(len(instance.dispatch.checkout) == cout)
+                self.assert_(len(instance.dispatch.checkin) == cin)
 
-        p = _pool()
+        p = self._queuepool_fixture()
         assert_listeners(p, 0, 0, 0, 0, 0)
 
         p.add_listener(ListenAll())
@@ -286,7 +519,7 @@ class PoolTest(PoolTestBase):
         del p
 
         snoop = ListenAll()
-        p = _pool(listeners=[snoop])
+        p = self._queuepool_fixture(listeners=[snoop])
         assert_listeners(p, 1, 1, 1, 1, 1)
 
         c = p.connect()
@@ -368,9 +601,8 @@ class PoolTest(PoolTestBase):
         c.close()
         snoop.assert_total(1, 1, 2, 2)
 
+    @testing.uses_deprecated(r".*Use event.listen")
     def test_listeners_callables(self):
-        dbapi = MockDBAPI()
-
         def connect(dbapi_con, con_record):
             counts[0] += 1
         def checkout(dbapi_con, con_record, con_proxy):
@@ -385,34 +617,30 @@ class PoolTest(PoolTestBase):
 
         for cls in (pool.QueuePool, pool.StaticPool):
             counts = [0, 0, 0]
-            def _pool(**kw):
-                return cls(creator=lambda: dbapi.connect('foo.db'),
-                                      use_threadlocal=False, **kw)
 
             def assert_listeners(p, total, conn, cout, cin):
                 for instance in (p, p.recreate()):
-                    self.assert_(len(instance.listeners) == total)
-                    self.assert_(len(instance._on_connect) == conn)
-                    self.assert_(len(instance._on_checkout) == cout)
-                    self.assert_(len(instance._on_checkin) == cin)
+                    eq_(len(instance.dispatch.connect), conn)
+                    eq_(len(instance.dispatch.checkout), cout)
+                    eq_(len(instance.dispatch.checkin), cin)
 
-            p = _pool()
+            p = self._queuepool_fixture()
             assert_listeners(p, 0, 0, 0, 0)
 
             p.add_listener(i_all)
             assert_listeners(p, 1, 1, 1, 1)
 
             p.add_listener(i_connect)
-            assert_listeners(p, 2, 2, 1, 1)
+            assert_listeners(p, 2, 1, 1, 1)
 
             p.add_listener(i_checkout)
-            assert_listeners(p, 3, 2, 2, 1)
+            assert_listeners(p, 3, 1, 1, 1)
 
             p.add_listener(i_checkin)
-            assert_listeners(p, 4, 2, 2, 2)
+            assert_listeners(p, 4, 1, 1, 1)
             del p
 
-            p = _pool(listeners=[i_all])
+            p = self._queuepool_fixture(listeners=[i_all])
             assert_listeners(p, 1, 1, 1, 1)
 
             c = p.connect()
@@ -424,19 +652,7 @@ class PoolTest(PoolTestBase):
             assert counts == [1, 2, 1]
             p.add_listener(i_checkin)
             c.close()
-            assert counts == [1, 2, 3]
-
-    def test_listener_after_oninit(self):
-        """Test that listeners are called after OnInit is removed"""
-        called = []
-        def listener(*args):
-            called.append(True)
-        listener.connect = listener
-        engine = create_engine(testing.db.url)
-        engine.pool.add_listener(listener)
-        engine.execute(select([1])).close()
-        assert called, "Listener not called on connect"
-
+            assert counts == [1, 2, 2]
 
 class QueuePoolTest(PoolTestBase):
 
@@ -447,8 +663,8 @@ class QueuePoolTest(PoolTestBase):
         self._do_testqueuepool(useclose=True)
 
     def _do_testqueuepool(self, useclose=False):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
-                           max_overflow=-1, use_threadlocal=False)
+        p = self._queuepool_fixture(pool_size=3,
+                           max_overflow=-1)
 
         def status(pool):
             tup = pool.size(), pool.checkedin(), pool.overflow(), \
@@ -499,8 +715,8 @@ class QueuePoolTest(PoolTestBase):
         assert not pool._refs
 
     def test_timeout(self):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
-                           max_overflow=0, use_threadlocal=False,
+        p = self._queuepool_fixture(pool_size=3,
+                           max_overflow=0,
                            timeout=2)
         c1 = p.connect()
         c2 = p.connect()
@@ -520,9 +736,10 @@ class QueuePoolTest(PoolTestBase):
         # wait for the timeout on queue.get().  the fix involves checking the
         # timeout again within the mutex, and if so, unlocking and throwing
         # them back to the start of do_get()
+        dbapi = MockDBAPI()
         p = pool.QueuePool(
-                creator = lambda: mock_dbapi.connect(delay=.05), 
-                pool_size = 2, 
+                creator = lambda: dbapi.connect(delay=.05),
+                pool_size = 2,
                 max_overflow = 1, use_threadlocal = False, timeout=3)
         timeouts = []
         def checkout():
@@ -542,7 +759,7 @@ class QueuePoolTest(PoolTestBase):
             th.start()
             threads.append(th)
         for th in threads:
-            th.join() 
+            th.join()
 
         assert len(timeouts) > 0
         for t in timeouts:
@@ -554,10 +771,11 @@ class QueuePoolTest(PoolTestBase):
     def _test_overflow(self, thread_count, max_overflow):
         gc_collect()
 
+        dbapi = MockDBAPI()
         def creator():
             time.sleep(.05)
-            return mock_dbapi.connect()
- 
+            return dbapi.connect()
+
         p = pool.QueuePool(creator=creator,
                            pool_size=3, timeout=2,
                            max_overflow=max_overflow)
@@ -579,21 +797,117 @@ class QueuePoolTest(PoolTestBase):
             threads.append(th)
         for th in threads:
             th.join()
- 
+
         self.assert_(max(peaks) <= max_overflow)
 
         lazy_gc()
         assert not pool._refs
- 
+
+    def test_waiters_handled(self):
+        """test that threads waiting for connections are
+        handled when the pool is replaced.
+
+        """
+        dbapi = MockDBAPI()
+        def creator():
+            return dbapi.connect()
+
+        success = []
+        for timeout in (None, 30):
+            for max_overflow in (0, -1, 3):
+                p = pool.QueuePool(creator=creator,
+                                   pool_size=2, timeout=timeout,
+                                   max_overflow=max_overflow)
+                def waiter(p):
+                    conn = p.connect()
+                    time.sleep(.5)
+                    success.append(True)
+                    conn.close()
+
+                time.sleep(.2)
+                c1 = p.connect()
+                c2 = p.connect()
+
+                for i in range(2):
+                    t = threading.Thread(target=waiter, args=(p, ))
+                    t.setDaemon(True) # so the tests dont hang if this fails
+                    t.start()
+
+                c1.invalidate()
+                c2.invalidate()
+                p2 = p._replace()
+        time.sleep(2)
+        eq_(len(success), 12)
+
+    @testing.requires.python26
+    def test_notify_waiters(self):
+        dbapi = MockDBAPI()
+        canary = []
+        def creator1():
+            canary.append(1)
+            return dbapi.connect()
+        def creator2():
+            canary.append(2)
+            return dbapi.connect()
+        p1 = pool.QueuePool(creator=creator1,
+                           pool_size=1, timeout=None,
+                           max_overflow=0)
+        p2 = pool.QueuePool(creator=creator2,
+                           pool_size=1, timeout=None,
+                           max_overflow=-1)
+        def waiter(p):
+            conn = p.connect()
+            time.sleep(.5)
+            conn.close()
+
+        c1 = p1.connect()
+
+        for i in range(5):
+            t = threading.Thread(target=waiter, args=(p1, ))
+            t.setDaemon(True)
+            t.start()
+        time.sleep(.5)
+        eq_(canary, [1])
+        p1._pool.abort(p2)
+        time.sleep(1)
+        eq_(canary, [1, 2, 2, 2, 2, 2])
+
+    def test_dispose_closes_pooled(self):
+        dbapi = MockDBAPI()
+        def creator():
+            return dbapi.connect()
+
+        p = pool.QueuePool(creator=creator,
+                           pool_size=2, timeout=None,
+                           max_overflow=0)
+        c1 = p.connect()
+        c2 = p.connect()
+        conns = [c1.connection, c2.connection]
+        c1.close()
+        eq_([c.closed for c in conns], [False, False])
+        p.dispose()
+        eq_([c.closed for c in conns], [True, False])
+
+        # currently, if a ConnectionFairy is closed
+        # after the pool has been disposed, there's no
+        # flag that states it should be invalidated
+        # immediately - it just gets returned to the
+        # pool normally...
+        c2.close()
+        eq_([c.closed for c in conns], [True, False])
+
+        # ...and that's the one we'll get back next.
+        c3 = p.connect()
+        assert c3.connection is conns[1]
+
     def test_no_overflow(self):
         self._test_overflow(40, 0)
- 
+
     def test_max_overflow(self):
         self._test_overflow(40, 5)
- 
+
     def test_mixed_close(self):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
-                           max_overflow=-1, use_threadlocal=True)
+        p = self._queuepool_fixture(pool_size=3, max_overflow=-1, use_threadlocal=True)
         c1 = p.connect()
         c2 = p.connect()
         assert c1 is c2
@@ -613,10 +927,10 @@ class QueuePoolTest(PoolTestBase):
         self._test_overflow_no_gc(False)
 
     def _test_overflow_no_gc(self, threadlocal):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=2,
-                           max_overflow=2, use_threadlocal=False)
+        p = self._queuepool_fixture(pool_size=2,
+                           max_overflow=2)
 
-        # disable weakref collection of the 
+        # disable weakref collection of the
         # underlying connections
         strong_refs = set()
         def _conn():
@@ -633,7 +947,7 @@ class QueuePoolTest(PoolTestBase):
         eq_(still_opened, 2)
 
     def test_weakref_kaboom(self):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
+        p = self._queuepool_fixture(pool_size=3,
                            max_overflow=-1, use_threadlocal=True)
         c1 = p.connect()
         c2 = p.connect()
@@ -653,7 +967,7 @@ class QueuePoolTest(PoolTestBase):
         ConnectionFairy with an ambiguous counter.  i.e. its not true
         reference counting."""
 
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
+        p = self._queuepool_fixture(pool_size=3,
                            max_overflow=-1, use_threadlocal=True)
         c1 = p.connect()
         c2 = p.connect()
@@ -664,10 +978,10 @@ class QueuePoolTest(PoolTestBase):
         self.assert_(p.checkedout() != 0)
         c2.close()
         self.assert_(p.checkedout() == 0)
- 
+
     def test_recycle(self):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=1,
-                           max_overflow=0, use_threadlocal=False,
+        p = self._queuepool_fixture(pool_size=1,
+                           max_overflow=0,
                            recycle=3)
         c1 = p.connect()
         c_id = id(c1.connection)
@@ -680,10 +994,7 @@ class QueuePoolTest(PoolTestBase):
         assert id(c3.connection) != c_id
 
     def test_invalidate(self):
-        dbapi = MockDBAPI()
-        p = pool.QueuePool(creator=lambda : dbapi.connect('foo.db'),
-                           pool_size=1, max_overflow=0,
-                           use_threadlocal=False)
+        p  = self._queuepool_fixture(pool_size=1, max_overflow=0)
         c1 = p.connect()
         c_id = c1.connection.id
         c1.close()
@@ -696,10 +1007,7 @@ class QueuePoolTest(PoolTestBase):
         assert c1.connection.id != c_id
 
     def test_recreate(self):
-        dbapi = MockDBAPI()
-        p = pool.QueuePool(creator=lambda : dbapi.connect('foo.db'),
-                           pool_size=1, max_overflow=0,
-                           use_threadlocal=False)
+        p = self._queuepool_fixture(pool_size=1, max_overflow=0)
         p2 = p.recreate()
         assert p2.size() == 1
         assert p2._use_threadlocal is False
@@ -710,10 +1018,7 @@ class QueuePoolTest(PoolTestBase):
         engine/dialect includes another layer of reconnect support for
         'database was lost' errors."""
 
-        dbapi = MockDBAPI()
-        p = pool.QueuePool(creator=lambda : dbapi.connect('foo.db'),
-                           pool_size=1, max_overflow=0,
-                           use_threadlocal=False)
+        dbapi, p = self._queuepool_dbapi_fixture(pool_size=1, max_overflow=0)
         c1 = p.connect()
         c_id = c1.connection.id
         c1.close()
@@ -727,10 +1032,7 @@ class QueuePoolTest(PoolTestBase):
         assert c1.connection.id != c_id
 
     def test_detach(self):
-        dbapi = MockDBAPI()
-        p = pool.QueuePool(creator=lambda : dbapi.connect('foo.db'),
-                           pool_size=1, max_overflow=0,
-                           use_threadlocal=False)
+        dbapi, p = self._queuepool_dbapi_fixture(pool_size=1, max_overflow=0)
         c1 = p.connect()
         c1.detach()
         c_id = c1.connection.id
@@ -747,8 +1049,7 @@ class QueuePoolTest(PoolTestBase):
         assert con.closed
 
     def test_threadfairy(self):
-        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
-                           max_overflow=-1, use_threadlocal=True)
+        p = self._queuepool_fixture(pool_size=3, max_overflow=-1, use_threadlocal=True)
         c1 = p.connect()
         c1.close()
         c2 = p.connect()
@@ -766,7 +1067,8 @@ class SingletonThreadPoolTest(PoolTestBase):
         """test that the pool's connections are OK after cleanup() has
         been called."""
 
-        p = pool.SingletonThreadPool(creator=mock_dbapi.connect,
+        dbapi = MockDBAPI()
+        p = pool.SingletonThreadPool(creator=dbapi.connect,
                 pool_size=3)
 
         if strong_refs:
@@ -800,6 +1102,24 @@ class SingletonThreadPoolTest(PoolTestBase):
             still_opened = len([c for c in sr if not c.closed])
             eq_(still_opened, 3)
 
+class AssertionPoolTest(PoolTestBase):
+    def test_connect_error(self):
+        dbapi = MockDBAPI()
+        p = pool.AssertionPool(creator = lambda: dbapi.connect('foo.db'))
+        c1 = p.connect()
+        assert_raises(AssertionError, p.connect)
+
+    def test_connect_multiple(self):
+        dbapi = MockDBAPI()
+        p = pool.AssertionPool(creator = lambda: dbapi.connect('foo.db'))
+        c1 = p.connect()
+        c1.close()
+        c2 = p.connect()
+        c2.close()
+
+        c3 = p.connect()
+        assert_raises(AssertionError, p.connect)
+
 class NullPoolTest(PoolTestBase):
     def test_reconnect(self):
         dbapi = MockDBAPI()
