@@ -4,10 +4,11 @@ import pickle
 
 from sqlalchemy import *
 from sqlalchemy.orm import *
-from sqlalchemy.orm.collections import collection
+from sqlalchemy.orm.collections import collection, attribute_mapped_collection
 from sqlalchemy.ext.associationproxy import *
 from sqlalchemy.ext.associationproxy import _AssociationList
 from test.lib import *
+from test.lib.testing import assert_raises_message
 from test.lib.util import gc_collect
 from sqlalchemy.sql import not_
 from test.lib import fixtures
@@ -640,8 +641,8 @@ class ProxyFactoryTest(ListTest):
                     )
 
         class Parent(object):
-            children = association_proxy('_children', 'name', 
-                        proxy_factory=CustomProxy, 
+            children = association_proxy('_children', 'name',
+                        proxy_factory=CustomProxy,
                         proxy_bulk_set=CustomProxy.extend
                     )
 
@@ -1006,7 +1007,8 @@ class PickleKeyFunc(object):
     def __call__(self, obj):
         return getattr(obj, self.name)
 
-class ComparatorTest(fixtures.MappedTest):
+class ComparatorTest(fixtures.MappedTest, AssertsCompiledSQL):
+    __dialect__ = 'default'
 
     run_inserts = 'once'
     run_deletes = None
@@ -1015,17 +1017,17 @@ class ComparatorTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        Table('userkeywords', metadata, 
+        Table('userkeywords', metadata,
           Column('keyword_id', Integer,ForeignKey('keywords.id'), primary_key=True),
           Column('user_id', Integer, ForeignKey('users.id'))
         )
-        Table('users', metadata, 
+        Table('users', metadata,
             Column('id', Integer,
               primary_key=True, test_needs_autoincrement=True),
             Column('name', String(64)),
             Column('singular_id', Integer, ForeignKey('singular.id'))
         )
-        Table('keywords', metadata, 
+        Table('keywords', metadata,
             Column('id', Integer,
               primary_key=True, test_needs_autoincrement=True),
             Column('keyword', String(64)),
@@ -1070,7 +1072,8 @@ class ComparatorTest(fixtures.MappedTest):
 
     @classmethod
     def setup_mappers(cls):
-        users, Keyword, UserKeyword, singular, userkeywords, User, keywords, Singular = (cls.tables.users,
+        users, Keyword, UserKeyword, singular, \
+            userkeywords, User, keywords, Singular = (cls.tables.users,
                                 cls.classes.Keyword,
                                 cls.classes.UserKeyword,
                                 cls.tables.singular,
@@ -1087,7 +1090,7 @@ class ComparatorTest(fixtures.MappedTest):
         })
 
         mapper(UserKeyword, userkeywords, properties={
-            'user' : relationship(User, backref='user_keywords'), 
+            'user' : relationship(User, backref='user_keywords'),
             'keyword' : relationship(Keyword)
         })
         mapper(Singular, singular, properties={
@@ -1280,3 +1283,87 @@ class ComparatorTest(fixtures.MappedTest):
 
         assert_raises(exceptions.InvalidRequestError, lambda : \
                       User.keywords != self.kw)
+
+    def test_join_separate_attr(self):
+        User = self.classes.User
+        self.assert_compile(
+            self.session.query(User).join(
+                        User.keywords.local_attr,
+                        User.keywords.remote_attr),
+            "SELECT users.id AS users_id, users.name AS users_name, "
+            "users.singular_id AS users_singular_id "
+            "FROM users JOIN userkeywords ON users.id = "
+            "userkeywords.user_id JOIN keywords ON keywords.id = "
+            "userkeywords.keyword_id"
+        )
+
+    def test_join_single_attr(self):
+        User = self.classes.User
+        self.assert_compile(
+            self.session.query(User).join(
+                        *User.keywords.attr),
+            "SELECT users.id AS users_id, users.name AS users_name, "
+            "users.singular_id AS users_singular_id "
+            "FROM users JOIN userkeywords ON users.id = "
+            "userkeywords.user_id JOIN keywords ON keywords.id = "
+            "userkeywords.keyword_id"
+        )
+
+class DictOfTupleUpdateTest(fixtures.TestBase):
+    def setup(self):
+        class B(object):
+            def __init__(self, key, elem):
+                self.key = key
+                self.elem = elem
+
+        class A(object):
+            elements = association_proxy("orig", "elem", creator=B)
+
+        m = MetaData()
+        a = Table('a', m, Column('id', Integer, primary_key=True))
+        b = Table('b', m, Column('id', Integer, primary_key=True),
+                    Column('aid', Integer, ForeignKey('a.id')))
+        mapper(A, a, properties={
+            'orig':relationship(B, collection_class=attribute_mapped_collection('key'))
+        })
+        mapper(B, b)
+        self.A = A
+        self.B = B
+
+    def test_update_one_elem_dict(self):
+        a1 = self.A()
+        a1.elements.update({("B", 3): 'elem2'})
+        eq_(a1.elements, {("B",3):'elem2'})
+
+    def test_update_multi_elem_dict(self):
+        a1 = self.A()
+        a1.elements.update({("B", 3): 'elem2', ("C", 4): "elem3"})
+        eq_(a1.elements, {("B",3):'elem2', ("C", 4): "elem3"})
+
+    def test_update_one_elem_list(self):
+        a1 = self.A()
+        a1.elements.update([(("B", 3), 'elem2')])
+        eq_(a1.elements, {("B",3):'elem2'})
+
+    def test_update_multi_elem_list(self):
+        a1 = self.A()
+        a1.elements.update([(("B", 3), 'elem2'), (("C", 4), "elem3")])
+        eq_(a1.elements, {("B",3):'elem2', ("C", 4): "elem3"})
+
+    def test_update_one_elem_varg(self):
+        a1 = self.A()
+        assert_raises_message(
+            ValueError,
+            "dictionary update sequence requires "
+            "2-element tuples",
+            a1.elements.update, (("B", 3), 'elem2')
+        )
+
+    def test_update_multi_elem_varg(self):
+        a1 = self.A()
+        assert_raises_message(
+            TypeError,
+            "update expected at most 1 arguments, got 2",
+            a1.elements.update,
+            (("B", 3), 'elem2'), (("C", 4), "elem3")
+        )
