@@ -9,30 +9,36 @@ UPDATE in the database.
 
 """
 
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from test.lib.testing import assert_raises, assert_raises_message
 from sqlalchemy import Integer, String, ForeignKey, Sequence, exc as sa_exc
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from test.lib.schema import Table
+from test.lib.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
     class_mapper, backref, sessionmaker, Session
 from sqlalchemy.orm import attributes, exc as orm_exc
-from sqlalchemy.test import testing
-from sqlalchemy.test.testing import eq_
-from test.orm import _base, _fixtures
+from test.lib import testing
+from test.lib.testing import eq_
+from test.lib import fixtures
+from test.orm import _fixtures
 
 class O2MCollectionTest(_fixtures.FixtureTest):
     run_inserts = None
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        Address, addresses, users, User = (cls.classes.Address,
+                                cls.tables.addresses,
+                                cls.tables.users,
+                                cls.classes.User)
+
         mapper(Address, addresses)
         mapper(User, users, properties = dict(
             addresses = relationship(Address, backref="user"),
         ))
 
-    @testing.resolve_artifact_names
     def test_collection_move_hitslazy(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         a2 = Address(email_address="address2")
@@ -50,8 +56,9 @@ class O2MCollectionTest(_fixtures.FixtureTest):
             u2.addresses.append(a3)
         self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_collection_move_preloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', addresses=[a1])
@@ -74,8 +81,9 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         assert a1 in u1.addresses
         assert a1 in u2.addresses
 
-    @testing.resolve_artifact_names
     def test_collection_move_notloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', addresses=[a1])
@@ -94,8 +102,9 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
 
-    @testing.resolve_artifact_names
     def test_collection_move_commitfirst(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', addresses=[a1])
@@ -112,14 +121,15 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         # backref fires
         assert a1.user is u2
 
-        # everything expires, no changes in 
+        # everything expires, no changes in
         # u1.addresses, so all is fine
         sess.commit()
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
 
-    @testing.resolve_artifact_names
     def test_scalar_move_preloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
 
         u1 = User(name='jack')
@@ -133,7 +143,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         u1.addresses
 
         # direct set - the "old" is "fetched",
-        # but only from the local session - not the 
+        # but only from the local session - not the
         # database, due to the PASSIVE_NO_FETCH flag.
         # this is a more fine grained behavior introduced
         # in 0.6
@@ -142,10 +152,12 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
 
-    @testing.resolve_artifact_names
     def test_plain_load_passive(self):
         """test that many-to-one set doesn't load the old value."""
 
+        User, Address = self.classes.User, self.classes.Address
+
+
         sess = sessionmaker()()
         u1 = User(name='jack')
         u2 = User(name='ed')
@@ -164,8 +176,9 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
 
-    @testing.resolve_artifact_names
     def test_set_none(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         u1 = User(name='jack')
         a1 = Address(email_address='a1')
@@ -182,8 +195,9 @@ class O2MCollectionTest(_fixtures.FixtureTest):
 
 
 
-    @testing.resolve_artifact_names
     def test_scalar_move_notloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
 
         u1 = User(name='jack')
@@ -193,7 +207,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         sess.add_all([u1, u2, a1])
         sess.commit()
 
-        # direct set - the fetching of the 
+        # direct set - the fetching of the
         # "old" u1 here allows the backref
         # to remove it from the addresses collection
         a1.user = u2
@@ -201,8 +215,9 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
 
-    @testing.resolve_artifact_names
     def test_scalar_move_commitfirst(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
 
         u1 = User(name='jack')
@@ -215,7 +230,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         # u1.addresses is loaded
         u1.addresses
 
-        # direct set - the fetching of the 
+        # direct set - the fetching of the
         # "old" u1 here allows the backref
         # to remove it from the addresses collection
         a1.user = u2
@@ -229,15 +244,20 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
     run_inserts = None
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        Address, addresses, users, User = (cls.classes.Address,
+                                cls.tables.addresses,
+                                cls.tables.users,
+                                cls.classes.User)
+
         mapper(Address, addresses)
         mapper(User, users, properties = {
             'address':relationship(Address, backref=backref("user"), uselist=False)
         })
 
-    @testing.resolve_artifact_names
     def test_collection_move_preloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', address=a1)
@@ -261,8 +281,9 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
         assert u1.address is a1
         assert u2.address is a1
 
-    @testing.resolve_artifact_names
     def test_scalar_move_preloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         a2 = Address(email_address="address1")
@@ -284,8 +305,9 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
         assert a1.user is u1
         assert a2.user is u1
 
-    @testing.resolve_artifact_names
     def test_collection_move_notloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', address=a1)
@@ -305,8 +327,9 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
         assert u1.address is None
         assert u2.address is a1
 
-    @testing.resolve_artifact_names
     def test_scalar_move_notloaded(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         a2 = Address(email_address="address1")
@@ -325,8 +348,9 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
         assert a1.user is u1
         assert a2.user is u1
 
-    @testing.resolve_artifact_names
     def test_collection_move_commitfirst(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', address=a1)
@@ -351,8 +375,9 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
         assert u1.address is None
         assert u2.address is a1
 
-    @testing.resolve_artifact_names
     def test_scalar_move_commitfirst(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         a2 = Address(email_address="address2")
@@ -383,15 +408,20 @@ class O2OScalarMoveTest(_fixtures.FixtureTest):
     run_inserts = None
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        Address, addresses, users, User = (cls.classes.Address,
+                                cls.tables.addresses,
+                                cls.tables.users,
+                                cls.classes.User)
+
         mapper(Address, addresses)
         mapper(User, users, properties = {
             'address':relationship(Address, uselist=False)
         })
 
-    @testing.resolve_artifact_names
     def test_collection_move_commitfirst(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', address=a1)
@@ -417,16 +447,22 @@ class O2OScalarOrphanTest(_fixtures.FixtureTest):
     run_inserts = None
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        Address, addresses, users, User = (cls.classes.Address,
+                                cls.tables.addresses,
+                                cls.tables.users,
+                                cls.classes.User)
+
         mapper(Address, addresses)
         mapper(User, users, properties = {
-            'address':relationship(Address, uselist=False, 
-                backref=backref('user', single_parent=True, cascade="all, delete-orphan"))
+            'address':relationship(Address, uselist=False,
+                backref=backref('user', single_parent=True,
+                                    cascade="all, delete-orphan"))
         })
 
-    @testing.resolve_artifact_names
     def test_m2o_event(self):
+        User, Address = self.classes.User, self.classes.Address
+
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', address=a1)
@@ -455,7 +491,7 @@ class M2MCollectionMoveTest(_fixtures.FixtureTest):
                                 cls.classes.Item)
 
         mapper(Item, items, properties={
-            'keywords':relationship(Keyword, secondary=item_keywords, 
+            'keywords':relationship(Keyword, secondary=item_keywords,
                                     backref='items')
         })
         mapper(Keyword, keywords)
@@ -559,17 +595,23 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
     run_inserts = None
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        keywords, items, item_keywords, Keyword, Item = (cls.tables.keywords,
+                                cls.tables.items,
+                                cls.tables.item_keywords,
+                                cls.classes.Keyword,
+                                cls.classes.Item)
+
         mapper(Item, items, properties={
-            'keyword':relationship(Keyword, secondary=item_keywords, 
-                                    uselist=False, 
+            'keyword':relationship(Keyword, secondary=item_keywords,
+                                    uselist=False,
                                     backref=backref("item", uselist=False))
         })
         mapper(Keyword, keywords)
 
-    @testing.resolve_artifact_names
     def test_collection_move_preloaded(self):
+        Item, Keyword = self.classes.Item, self.classes.Keyword
+
         sess = sessionmaker()()
 
         k1 = Keyword(name='k1')
@@ -590,8 +632,9 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
         assert i1.keyword is k1
         assert i2.keyword is k1
 
-    @testing.resolve_artifact_names
     def test_collection_move_notloaded(self):
+        Item, Keyword = self.classes.Item, self.classes.Keyword
+
         sess = sessionmaker()()
 
         k1 = Keyword(name='k1')
@@ -608,8 +651,9 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
         assert i1.keyword is None
         assert i2.keyword is k1
 
-    @testing.resolve_artifact_names
     def test_collection_move_commit(self):
+        Item, Keyword = self.classes.Item, self.classes.Keyword
+
         sess = sessionmaker()()
 
         k1 = Keyword(name='k1')
@@ -629,3 +673,65 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
         sess.commit()
         assert i1.keyword is None
         assert i2.keyword is k1
+
+class O2MStaleBackrefTest(_fixtures.FixtureTest):
+    run_inserts = None
+
+    @classmethod
+    def setup_mappers(cls):
+        Address, addresses, users, User = (cls.classes.Address,
+                                cls.tables.addresses,
+                                cls.tables.users,
+                                cls.classes.User)
+
+        mapper(Address, addresses)
+        mapper(User, users, properties = dict(
+            addresses = relationship(Address, backref="user"),
+        ))
+
+
+    def test_backref_pop_m2o(self):
+        User, Address = self.classes.User, self.classes.Address
+
+        u1 = User()
+        u2 = User()
+        a1 = Address()
+        u1.addresses.append(a1)
+        u2.addresses.append(a1)
+
+        # events haven't updated
+        # u1.addresses here.
+        u1.addresses.remove(a1)
+
+        assert a1.user is u2
+        assert a1 in u2.addresses
+
+class M2MStaleBackrefTest(_fixtures.FixtureTest):
+    run_inserts = None
+
+    @classmethod
+    def setup_mappers(cls):
+        keywords, items, item_keywords, Keyword, Item = (cls.tables.keywords,
+                                cls.tables.items,
+                                cls.tables.item_keywords,
+                                cls.classes.Keyword,
+                                cls.classes.Item)
+
+        mapper(Item, items, properties={
+            'keywords':relationship(Keyword, secondary=item_keywords,
+                                    backref='items')
+        })
+        mapper(Keyword, keywords)
+
+    def test_backref_pop_m2m(self):
+        Keyword, Item = self.classes.Keyword, self.classes.Item
+
+        k1 = Keyword()
+        k2 = Keyword()
+        i1 = Item()
+        k1.items.append(i1)
+        k2.items.append(i1)
+        k2.items.append(i1)
+        i1.keywords = []
+        k2.items.remove(i1)
+        assert len(k2.items) == 0
