@@ -1,4 +1,4 @@
-from sqlalchemy.test.testing import eq_
+from test.lib.testing import eq_
 import sys
 from operator import and_
 
@@ -6,13 +6,14 @@ import sqlalchemy.orm.collections as collections
 from sqlalchemy.orm.collections import collection
 
 import sqlalchemy as sa
-from sqlalchemy.test import testing
+from test.lib import testing
 from sqlalchemy import Integer, String, ForeignKey, text
-from sqlalchemy.test.schema import Table, Column
+from test.lib.schema import Table, Column
 from sqlalchemy import util, exc as sa_exc
-from sqlalchemy.orm import create_session, mapper, relationship, attributes
-from test.orm import _base
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
+from sqlalchemy.orm import create_session, mapper, relationship, \
+    attributes, instrumentation
+from test.lib import fixtures
+from test.lib.testing import eq_, assert_raises, assert_raises_message
 
 class Canary(sa.orm.interfaces.AttributeExtension):
     def __init__(self):
@@ -37,7 +38,7 @@ class Canary(sa.orm.interfaces.AttributeExtension):
         self.append(obj, value, None)
         return value
 
-class CollectionsTest(_base.ORMTest):
+class CollectionsTest(fixtures.ORMTest):
     class Entity(object):
         def __init__(self, a=None, b=None, c=None):
             self.a = a
@@ -48,11 +49,11 @@ class CollectionsTest(_base.ORMTest):
 
     @classmethod
     def setup_class(cls):
-        attributes.register_class(cls.Entity)
+        instrumentation.register_class(cls.Entity)
 
     @classmethod
     def teardown_class(cls):
-        attributes.unregister_class(cls.Entity)
+        instrumentation.unregister_class(cls.Entity)
         super(CollectionsTest, cls).teardown_class()
 
     _entity_id = 1
@@ -75,7 +76,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -116,7 +117,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -317,7 +318,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -485,7 +486,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -743,7 +744,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -853,7 +854,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -975,7 +976,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -1174,7 +1175,7 @@ class CollectionsTest(_base.ORMTest):
             pass
 
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=typecallable, useobject=True)
 
@@ -1310,7 +1311,7 @@ class CollectionsTest(_base.ORMTest):
         class Foo(object):
             pass
         canary = Canary()
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary,
                                    typecallable=Custom, useobject=True)
 
@@ -1380,7 +1381,7 @@ class CollectionsTest(_base.ORMTest):
 
         canary = Canary()
         creator = self.entity_maker
-        attributes.register_class(Foo)
+        instrumentation.register_class(Foo)
         attributes.register_attribute(Foo, 'attr', uselist=True, extension=canary, useobject=True)
 
         obj = Foo()
@@ -1407,7 +1408,7 @@ class CollectionsTest(_base.ORMTest):
         obj.attr[0] = e3
         self.assert_(e3 in canary.data)
 
-class DictHelpersTest(_base.MappedTest):
+class DictHelpersTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
@@ -1424,18 +1425,22 @@ class DictHelpersTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Parent(_base.BasicEntity):
+        class Parent(cls.Basic):
             def __init__(self, label=None):
                 self.label = label
 
-        class Child(_base.BasicEntity):
+        class Child(cls.Basic):
             def __init__(self, a=None, b=None, c=None):
                 self.a = a
                 self.b = b
                 self.c = c
 
-    @testing.resolve_artifact_names
     def _test_scalar_mapped(self, collection_class):
+        parents, children, Parent, Child = (self.tables.parents,
+                                self.tables.children,
+                                self.classes.Parent,
+                                self.classes.Child)
+
         mapper(Child, children)
         mapper(Parent, parents, properties={
             'children': relationship(Child, collection_class=collection_class,
@@ -1493,8 +1498,12 @@ class DictHelpersTest(_base.MappedTest):
         self.assert_(len(list(collections.collection_adapter(p.children))) == 0)
 
 
-    @testing.resolve_artifact_names
     def _test_composite_mapped(self, collection_class):
+        parents, children, Parent, Child = (self.tables.parents,
+                                self.tables.children,
+                                self.classes.Parent,
+                                self.classes.Child)
+
         mapper(Child, children)
         mapper(Parent, parents, properties={
             'children': relationship(Child, collection_class=collection_class,
@@ -1550,19 +1559,18 @@ class DictHelpersTest(_base.MappedTest):
 
         class Foo(BaseObject):
             __tablename__ = "foo"
-            id = Column(Integer(), primary_key=True, test_needs_autoincrement=True)
+            id = Column(Integer(), primary_key=True)
             bar_id = Column(Integer, ForeignKey('bar.id'))
 
-        class Bar(BaseObject):
-            __tablename__ = "bar"
-            id = Column(Integer(), primary_key=True, test_needs_autoincrement=True)
-            foos = relationship(Foo, collection_class=collections.column_mapped_collection(Foo.id))
-            foos2 = relationship(Foo, collection_class=collections.column_mapped_collection((Foo.id, Foo.bar_id)))
+        for spec, obj, expected in (
+            (Foo.id, Foo(id=3), 3),
+            ((Foo.id, Foo.bar_id), Foo(id=3, bar_id=12), (3, 12))
+        ):
+            eq_(
+                collections.column_mapped_collection(spec)().keyfunc(obj),
+                expected
+            )
 
-        eq_(Bar.foos.property.collection_class().keyfunc(Foo(id=3)), 3)
-        eq_(Bar.foos2.property.collection_class().keyfunc(Foo(id=3, bar_id=12)), (3, 12))
-
-    @testing.resolve_artifact_names
     def test_column_mapped_assertions(self):
         assert_raises_message(sa_exc.ArgumentError,
                               "Column-based expression object expected "
@@ -1575,14 +1583,16 @@ class DictHelpersTest(_base.MappedTest):
                               text('a'))
 
 
-    @testing.resolve_artifact_names
     def test_column_mapped_collection(self):
+        children = self.tables.children
+
         collection_class = collections.column_mapped_collection(
             children.c.a)
         self._test_scalar_mapped(collection_class)
 
-    @testing.resolve_artifact_names
     def test_column_mapped_collection2(self):
+        children = self.tables.children
+
         collection_class = collections.column_mapped_collection(
             (children.c.a, children.c.b))
         self._test_composite_mapped(collection_class)
@@ -1603,7 +1613,77 @@ class DictHelpersTest(_base.MappedTest):
         collection_class = lambda: Ordered2(lambda v: (v.a, v.b))
         self._test_composite_mapped(collection_class)
 
-class CustomCollectionsTest(_base.MappedTest):
+class ColumnMappedWSerialize(fixtures.MappedTest):
+    """test the column_mapped_collection serializer against
+    multi-table and indirect table edge cases, including
+    serialization."""
+
+    run_create_tables = run_deletes = None
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('foo', metadata,
+            Column('id', Integer(), primary_key=True),
+            Column('b', String(128))
+        )
+        Table('bar', metadata,
+            Column('id', Integer(), primary_key=True),
+            Column('foo_id', Integer, ForeignKey('foo.id')),
+            Column('bat_id', Integer),
+            schema="x"
+        )
+    @classmethod
+    def setup_classes(cls):
+        class Foo(cls.Basic):
+            pass
+        class Bar(Foo):
+            pass
+
+    def test_indirect_table_column_mapped(self):
+        Foo = self.classes.Foo
+        Bar = self.classes.Bar
+        bar = self.tables["x.bar"]
+        mapper(Foo, self.tables.foo, properties={
+            "foo_id":self.tables.foo.c.id
+        })
+        mapper(Bar, bar, inherits=Foo, properties={
+            "bar_id":bar.c.id,
+        })
+
+        bar_spec = Bar(foo_id=1, bar_id=2, bat_id=3)
+        self._run_test([
+            (Foo.foo_id, bar_spec, 1),
+            ((Bar.bar_id, Bar.bat_id), bar_spec, (2, 3)),
+            (Bar.foo_id, bar_spec, 1),
+            (bar.c.id, bar_spec, 2),
+        ])
+
+    def test_selectable_column_mapped(self):
+        from sqlalchemy import select
+        s = select([self.tables.foo]).alias()
+        Foo = self.classes.Foo
+        mapper(Foo, s)
+        self._run_test([
+            (Foo.b, Foo(b=5), 5),
+            (s.c.b, Foo(b=5), 5)
+        ])
+
+    def _run_test(self, specs):
+        from test.lib.util import picklers
+        for spec, obj, expected in specs:
+            coll = collections.column_mapped_collection(spec)()
+            eq_(
+                coll.keyfunc(obj),
+                expected
+            )
+            # ensure we do the right thing with __reduce__
+            for loads, dumps in picklers():
+                c2 = loads(dumps(coll))
+                eq_(c2.keyfunc(obj), expected)
+                c3 = loads(dumps(c2))
+                eq_(c3.keyfunc(obj), expected)
+
+class CustomCollectionsTest(fixtures.MappedTest):
     """test the integration of collections with mapped classes."""
 
     @classmethod
@@ -1617,8 +1697,9 @@ class CustomCollectionsTest(_base.MappedTest):
                      ForeignKey('sometable.col1')),
               Column('data', String(20)))
 
-    @testing.resolve_artifact_names
     def test_basic(self):
+        someothertable, sometable = self.tables.someothertable, self.tables.sometable
+
         class MyList(list):
             pass
         class Foo(object):
@@ -1633,9 +1714,11 @@ class CustomCollectionsTest(_base.MappedTest):
         f = Foo()
         assert isinstance(f.bars, MyList)
 
-    @testing.resolve_artifact_names
     def test_lazyload(self):
         """test that a 'set' can be used as a collection and can lazyload."""
+
+        someothertable, sometable = self.tables.someothertable, self.tables.sometable
+
         class Foo(object):
             pass
         class Bar(object):
@@ -1655,10 +1738,12 @@ class CustomCollectionsTest(_base.MappedTest):
         assert len(list(f.bars)) == 2
         f.bars.clear()
 
-    @testing.resolve_artifact_names
     def test_dict(self):
         """test that a 'dict' can be used as a collection and can lazyload."""
 
+        someothertable, sometable = self.tables.someothertable, self.tables.sometable
+
+
         class Foo(object):
             pass
         class Bar(object):
@@ -1687,10 +1772,12 @@ class CustomCollectionsTest(_base.MappedTest):
         assert len(list(f.bars)) == 2
         f.bars.clear()
 
-    @testing.resolve_artifact_names
     def test_dict_wrapper(self):
         """test that the supplied 'dict' wrapper can be used as a collection and can lazyload."""
 
+        someothertable, sometable = self.tables.someothertable, self.tables.sometable
+
+
         class Foo(object):
             pass
         class Bar(object):
@@ -1762,8 +1849,9 @@ class CustomCollectionsTest(_base.MappedTest):
 
         self._test_list(ListLike)
 
-    @testing.resolve_artifact_names
     def _test_list(self, listcls):
+        someothertable, sometable = self.tables.someothertable, self.tables.sometable
+
         class Parent(object):
             pass
         class Child(object):
@@ -1884,8 +1972,9 @@ class CustomCollectionsTest(_base.MappedTest):
         assert control == p.children
         assert control == list(p.children)
 
-    @testing.resolve_artifact_names
     def test_custom(self):
+        someothertable, sometable = self.tables.someothertable, self.tables.sometable
+
         class Parent(object):
             pass
         class Child(object):
@@ -1937,7 +2026,7 @@ class CustomCollectionsTest(_base.MappedTest):
         assert len(o) == 3
 
 
-class InstrumentationTest(_base.ORMTest):
+class InstrumentationTest(fixtures.ORMTest):
     def test_uncooperative_descriptor_in_sweep(self):
         class DoNotTouch(object):
             def __get__(self, obj, owner):
