@@ -1,22 +1,27 @@
 """Attribute/instance expiration, deferral of attributes, etc."""
 
-from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
-from sqlalchemy.test.util import gc_collect
+from test.lib.testing import eq_, assert_raises, assert_raises_message
+from test.lib.util import gc_collect
 import sqlalchemy as sa
-from sqlalchemy.test import testing
+from test.lib import testing
 from sqlalchemy import Integer, String, ForeignKey, exc as sa_exc
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from test.lib.schema import Table
+from test.lib.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
                         attributes, deferred, exc as orm_exc, defer, undefer,\
                         strategies, state, lazyload, backref, Session
-from test.orm import _base, _fixtures
+from test.lib import fixtures
+from test.orm import _fixtures
 
 
 class ExpireTest(_fixtures.FixtureTest):
 
-    @testing.resolve_artifact_names
     def test_expire(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user'),
             })
@@ -54,18 +59,20 @@ class ExpireTest(_fixtures.FixtureTest):
             assert u.name == 'jack'
         self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_persistence_check(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         s = create_session()
         u = s.query(User).get(7)
         s.expunge_all()
 
-        assert_raises_message(sa_exc.InvalidRequestError, 
+        assert_raises_message(sa_exc.InvalidRequestError,
                         r"is not persistent within this Session", s.expire, u)
 
-    @testing.resolve_artifact_names
     def test_get_refreshes(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         s = create_session(autocommit=False)
         u = s.query(User).get(10)
@@ -81,6 +88,13 @@ class ExpireTest(_fixtures.FixtureTest):
             u = s.query(User).get(10)  # expire flag reset, so not expired
         self.assert_sql_count(testing.db, go, 0)
 
+    def test_get_on_deleted_expunges(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+
         s.expire_all()
         s.execute(users.delete().where(User.id==10))
 
@@ -89,31 +103,54 @@ class ExpireTest(_fixtures.FixtureTest):
         assert s.query(User).get(10) is None
         assert u not in s # and expunges
 
-        # trick the "deleted" flag so we can re-add for the sake
-        # of this test
-        del attributes.instance_state(u).deleted
+    def test_refresh_on_deleted_raises(self):
+        users, User = self.tables.users, self.classes.User
 
-        # add it back
-        s.add(u)
-        # nope, raises ObjectDeletedError
-        assert_raises(sa.orm.exc.ObjectDeletedError, getattr, u, 'name')
+        mapper(User, users)
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+        s.expire_all()
 
-        # do a get()/remove u from session again
+        s.expire_all()
+        s.execute(users.delete().where(User.id==10))
+
+        # raises ObjectDeletedError
+        assert_raises_message(
+            sa.orm.exc.ObjectDeletedError,
+            "Instance '<User at .*?>' has been "
+            "deleted, or its row is otherwise not present.",
+            getattr, u, 'name'
+        )
+
+    def test_rollback_undoes_expunge_from_deleted(self):
+        users, User = self.tables.users, self.classes.User
+
+        mapper(User, users)
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+        s.expire_all()
+        s.execute(users.delete().where(User.id==10))
+
+        # do a get()/remove u from session
         assert s.query(User).get(10) is None
         assert u not in s
 
         s.rollback()
 
         assert u in s
-        # but now its back, rollback has occurred, the _remove_newly_deleted
-        # is reverted
+        # but now its back, rollback has occurred, the
+        # _remove_newly_deleted is reverted
         eq_(u.name, 'chuck')
 
-    @testing.resolve_artifact_names
     def test_deferred(self):
-        """test that unloaded, deferred attributes aren't included in the expiry list."""
+        """test that unloaded, deferred attributes aren't included in the
+        expiry list."""
 
-        mapper(Order, orders, properties={'description':deferred(orders.c.description)})
+        Order, orders = self.classes.Order, self.tables.orders
+
+
+        mapper(Order, orders, properties={
+                    'description':deferred(orders.c.description)})
 
         s = create_session()
         o1 = s.query(Order).first()
@@ -123,50 +160,79 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'description' not in o1.__dict__
         assert o1.description
 
-    @testing.resolve_artifact_names
-    def test_lazyload_autoflushes(self):
+    def test_deferred_notfound(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users, properties={
-            'addresses':relationship(Address, order_by=addresses.c.email_address)
+            'name':deferred(users.c.name)
+        })
+        s = create_session(autocommit=False)
+        u = s.query(User).get(10)
+
+        assert 'name' not in u.__dict__
+        s.execute(users.delete().where(User.id==10))
+        assert_raises_message(
+            sa.orm.exc.ObjectDeletedError,
+            "Instance '<User at .*?>' has been "
+            "deleted, or its row is otherwise not present.",
+            getattr, u, 'name'
+        )
+
+    def test_lazyload_autoflushes(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        mapper(User, users, properties={
+            'addresses':relationship(Address,
+                    order_by=addresses.c.email_address)
         })
         mapper(Address, addresses)
         s = create_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
         adlist = u.addresses
         eq_(adlist, [
-            Address(email_address='ed@bettyboop.com'), 
+            Address(email_address='ed@bettyboop.com'),
             Address(email_address='ed@lala.com'),
-            Address(email_address='ed@wood.com'), 
+            Address(email_address='ed@wood.com'),
         ])
         a1 = u.addresses[2]
         a1.email_address = 'aaaaa'
         s.expire(u, ['addresses'])
         eq_(u.addresses, [
-            Address(email_address='aaaaa'), 
-            Address(email_address='ed@bettyboop.com'), 
+            Address(email_address='aaaaa'),
+            Address(email_address='ed@bettyboop.com'),
             Address(email_address='ed@lala.com'),
         ])
 
-    @testing.resolve_artifact_names
     def test_refresh_collection_exception(self):
-        """test graceful failure for currently unsupported 
+        """test graceful failure for currently unsupported
         immediate refresh of a collection"""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={
             'addresses':relationship(Address, order_by=addresses.c.email_address)
         })
         mapper(Address, addresses)
         s = create_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
-        assert_raises_message(sa_exc.InvalidRequestError, 
-                        "properties specified for refresh", 
+        assert_raises_message(sa_exc.InvalidRequestError,
+                        "properties specified for refresh",
                         s.refresh, u, ['addresses'])
 
         # in contrast to a regular query with no columns
-        assert_raises_message(sa_exc.InvalidRequestError, 
+        assert_raises_message(sa_exc.InvalidRequestError,
                         "no columns with which to SELECT", s.query().all)
 
-    @testing.resolve_artifact_names
     def test_refresh_cancels_expire(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         s = create_session()
         u = s.query(User).get(7)
@@ -178,8 +244,9 @@ class ExpireTest(_fixtures.FixtureTest):
             eq_(u.name, 'jack')
         self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_expire_doesntload_on_set(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
 
         sess = create_session()
@@ -193,8 +260,9 @@ class ExpireTest(_fixtures.FixtureTest):
         sess.expunge_all()
         assert sess.query(User).get(7).name == 'somenewname'
 
-    @testing.resolve_artifact_names
     def test_no_session(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         sess = create_session()
         u = sess.query(User).get(7)
@@ -203,8 +271,9 @@ class ExpireTest(_fixtures.FixtureTest):
         sess.expunge(u)
         assert_raises(orm_exc.DetachedInstanceError, getattr, u, 'name')
 
-    @testing.resolve_artifact_names
     def test_pending_raises(self):
+        users, User = self.tables.users, self.classes.User
+
         # this was the opposite in 0.4, but the reasoning there seemed off.
         # expiring a pending instance makes no sense, so should raise
         mapper(User, users)
@@ -213,8 +282,9 @@ class ExpireTest(_fixtures.FixtureTest):
         sess.add(u)
         assert_raises(sa_exc.InvalidRequestError, sess.expire, u, ['name'])
 
-    @testing.resolve_artifact_names
     def test_no_instance_key(self):
+        User, users = self.classes.User, self.tables.users
+
         # this tests an artificial condition such that
         # an instance is pending, but has expired attributes.  this
         # is actually part of a larger behavior when postfetch needs to
@@ -230,8 +300,9 @@ class ExpireTest(_fixtures.FixtureTest):
         sess.add(u)
         assert u.name == 'jack'
 
-    @testing.resolve_artifact_names
     def test_no_instance_key_no_pk(self):
+        users, User = self.tables.users, self.classes.User
+
         # same as test_no_instance_key, but the PK columns
         # are absent.  ensure an error is raised.
         mapper(User, users)
@@ -246,10 +317,12 @@ class ExpireTest(_fixtures.FixtureTest):
         assert_raises(sa_exc.InvalidRequestError, getattr, u, 'name')
 
 
-    @testing.resolve_artifact_names
     def test_expire_preserves_changes(self):
         """test that the expire load operation doesn't revert post-expire changes"""
 
+        Order, orders = self.classes.Order, self.tables.orders
+
+
         mapper(Order, orders)
         sess = create_session()
         o = sess.query(Order).get(3)
@@ -291,9 +364,11 @@ class ExpireTest(_fixtures.FixtureTest):
         assert o.isopen == 1
         assert o.description == 'another new description'
 
-    @testing.resolve_artifact_names
     def test_expire_committed(self):
         """test that the committed state of the attribute receives the most recent DB data"""
+
+        orders, Order = self.tables.orders, self.classes.Order
+
         mapper(Order, orders)
 
         sess = create_session()
@@ -307,8 +382,12 @@ class ExpireTest(_fixtures.FixtureTest):
             sess.flush()
         self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_expire_cascade(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, cascade="all, refresh-expire")
         })
@@ -321,8 +400,12 @@ class ExpireTest(_fixtures.FixtureTest):
         s.expire(u)
         assert u.addresses[0].email_address == 'ed@wood.com'
 
-    @testing.resolve_artifact_names
     def test_refresh_cascade(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, cascade="all, refresh-expire")
         })
@@ -351,8 +434,12 @@ class ExpireTest(_fixtures.FixtureTest):
         cascade = 'save-update, refresh-expire'
         self._test_cascade_to_pending(cascade, False)
 
-    @testing.resolve_artifact_names
     def _test_cascade_to_pending(self, cascade, expire_or_refresh):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, cascade=cascade)
         })
@@ -375,8 +462,12 @@ class ExpireTest(_fixtures.FixtureTest):
         assert a not in u.addresses
         s.flush()
 
-    @testing.resolve_artifact_names
     def test_expired_lazy(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user'),
             })
@@ -397,8 +488,12 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' in u.__dict__
         assert 'addresses' in u.__dict__
 
-    @testing.resolve_artifact_names
     def test_expired_eager(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user', lazy='joined'),
             })
@@ -432,8 +527,12 @@ class ExpireTest(_fixtures.FixtureTest):
         # together when eager load used with Query
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_relationship_changes_preserved(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user', lazy='joined'),
             })
@@ -448,8 +547,12 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' in u.__dict__
         assert len(u.addresses) == 2
 
-    @testing.resolve_artifact_names
     def test_joinedload_props_dontload(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         # relationships currently have to load separately from scalar instances.
         # the use case is: expire "addresses".  then access it.  lazy load
         # fires off to load "addresses", but needs foreign key or primary key
@@ -472,8 +575,9 @@ class ExpireTest(_fixtures.FixtureTest):
         u.addresses
         assert 'addresses' in u.__dict__
 
-    @testing.resolve_artifact_names
     def test_expire_synonym(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users, properties={
             'uname': sa.orm.synonym('name')
         })
@@ -497,8 +601,9 @@ class ExpireTest(_fixtures.FixtureTest):
         #    users.update(users.c.id==7).execute(name='jack3')
         #    assert u.uname == 'jack3'
 
-    @testing.resolve_artifact_names
     def test_partial_expire(self):
+        orders, Order = self.tables.orders, self.classes.Order
+
         mapper(Order, orders)
 
         sess = create_session()
@@ -542,8 +647,12 @@ class ExpireTest(_fixtures.FixtureTest):
             assert o.isopen == 5
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_partial_expire_lazy(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user'),
             })
@@ -587,8 +696,12 @@ class ExpireTest(_fixtures.FixtureTest):
             assert u.name == 'ed'
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_partial_expire_eager(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user', lazy='joined'),
             })
@@ -628,8 +741,12 @@ class ExpireTest(_fixtures.FixtureTest):
         # doing it that way right now
         #self.assert_sql_count(testing.db, go, 0)
 
-    @testing.resolve_artifact_names
     def test_relationships_load_on_query(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user'),
             })
@@ -649,8 +766,9 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' in u.__dict__
         assert 'addresses' in u.__dict__
 
-    @testing.resolve_artifact_names
     def test_partial_expire_deferred(self):
+        orders, Order = self.tables.orders, self.classes.Order
+
         mapper(Order, orders, properties={
             'description': sa.orm.deferred(orders.c.description)
         })
@@ -716,8 +834,12 @@ class ExpireTest(_fixtures.FixtureTest):
             assert o.isopen == 1
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_joinedload_query_refreshes(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user', lazy='joined'),
             })
@@ -733,10 +855,14 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'addresses' in u.__dict__
         assert len(u.addresses) == 3
 
-    @testing.resolve_artifact_names
     def test_expire_all(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
-            'addresses':relationship(Address, backref='user', lazy='joined', 
+            'addresses':relationship(Address, backref='user', lazy='joined',
                                     order_by=addresses.c.id),
             })
         mapper(Address, addresses)
@@ -754,10 +880,12 @@ class ExpireTest(_fixtures.FixtureTest):
         eq_(self.static.user_address_result, userlist)
         assert len(list(sess)) == 9
 
-    @testing.resolve_artifact_names
     def test_state_change_col_to_deferred(self):
         """Behavioral test to verify the current activity of loader callables."""
 
+        users, User = self.tables.users, self.classes.User
+
+
         mapper(User, users)
 
         sess = create_session()
@@ -801,17 +929,19 @@ class ExpireTest(_fixtures.FixtureTest):
         sess.expire(u1)
         assert 'name' in attributes.instance_state(u1).callables
 
-    @testing.resolve_artifact_names
     def test_state_deferred_to_col(self):
         """Behavioral test to verify the current activity of loader callables."""
 
+        users, User = self.tables.users, self.classes.User
+
+
         mapper(User, users, properties={'name':deferred(users.c.name)})
 
         sess = create_session()
         u1 = sess.query(User).options(undefer(User.name)).first()
         assert 'name' not in attributes.instance_state(u1).callables
 
-        # mass expire, the attribute was loaded, 
+        # mass expire, the attribute was loaded,
         # the attribute gets the callable
         sess.expire(u1)
         assert isinstance(
@@ -824,7 +954,7 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' not in attributes.instance_state(u1).callables
 
         # mass expire, attribute was loaded but then deleted,
-        # the callable goes away - the state wants to flip 
+        # the callable goes away - the state wants to flip
         # it back to its "deferred" loader.
         sess.expunge_all()
         u1 = sess.query(User).options(undefer(User.name)).first()
@@ -841,10 +971,15 @@ class ExpireTest(_fixtures.FixtureTest):
                     state.InstanceState
                 )
 
-    @testing.resolve_artifact_names
     def test_state_noload_to_lazy(self):
         """Behavioral test to verify the current activity of loader callables."""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={'addresses':relationship(Address, lazy='noload')})
         mapper(Address, addresses)
 
@@ -879,14 +1014,12 @@ class ExpireTest(_fixtures.FixtureTest):
 
 
 
-class PolymorphicExpireTest(_base.MappedTest):
+class PolymorphicExpireTest(fixtures.MappedTest):
     run_inserts = 'once'
     run_deletes = None
 
     @classmethod
     def define_tables(cls, metadata):
-        global people, engineers, Person, Engineer
-
         people = Table('people', metadata,
            Column('person_id', Integer, primary_key=True,
                   test_needs_autoincrement=True),
@@ -902,14 +1035,15 @@ class PolymorphicExpireTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Person(_base.ComparableEntity):
+        class Person(cls.Basic):
             pass
         class Engineer(Person):
             pass
 
     @classmethod
-    @testing.resolve_artifact_names
     def insert_data(cls):
+        people, engineers = cls.tables.people, cls.tables.engineers
+
         people.insert().execute(
             {'person_id':1, 'name':'person1', 'type':'person'},
             {'person_id':2, 'name':'engineer1', 'type':'engineer'},
@@ -921,13 +1055,20 @@ class PolymorphicExpireTest(_base.MappedTest):
         )
 
     @classmethod
-    @testing.resolve_artifact_names
     def setup_mappers(cls):
+        Person, people, engineers, Engineer = (cls.classes.Person,
+                                cls.tables.people,
+                                cls.tables.engineers,
+                                cls.classes.Engineer)
+
         mapper(Person, people, polymorphic_on=people.c.type, polymorphic_identity='person')
         mapper(Engineer, engineers, inherits=Person, polymorphic_identity='engineer')
 
-    @testing.resolve_artifact_names
     def test_poly_deferred(self):
+        Person, people, Engineer = (self.classes.Person,
+                                self.tables.people,
+                                self.classes.Engineer)
+
 
         sess = create_session()
         [p1, e1, e2] = sess.query(Person).order_by(people.c.person_id).all()
@@ -963,8 +1104,9 @@ class PolymorphicExpireTest(_base.MappedTest):
         self.assert_sql_count(testing.db, go, 2)
         eq_(Engineer.name.get_history(e1), (['new engineer name'],(), ['engineer1']))
 
-    @testing.resolve_artifact_names
     def test_no_instance_key(self):
+        Engineer = self.classes.Engineer
+
 
         sess = create_session()
         e1 = sess.query(Engineer).get(2)
@@ -976,8 +1118,9 @@ class PolymorphicExpireTest(_base.MappedTest):
         sess.add(e1)
         assert e1.name == 'engineer1'
 
-    @testing.resolve_artifact_names
     def test_no_instance_key(self):
+        Engineer = self.classes.Engineer
+
         # same as test_no_instance_key, but the PK columns
         # are absent.  ensure an error is raised.
         sess = create_session()
@@ -997,8 +1140,12 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
     run_setup_mappers = None
     run_inserts = None
 
-    @testing.resolve_artifact_names
     def test_expired_pending(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user'),
             })
@@ -1017,7 +1164,7 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
         # which attach to u1 will expect to be "pending"
         sess.expire(u1, ['addresses'])
 
-        # attach an Address.  now its "pending" 
+        # attach an Address.  now its "pending"
         # in user.addresses
         a2 = Address(email_address='a2')
         a2.user = u1
@@ -1039,8 +1186,12 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
 
 class RefreshTest(_fixtures.FixtureTest):
 
-    @testing.resolve_artifact_names
     def test_refresh(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses), backref='user')
         })
@@ -1075,16 +1226,18 @@ class RefreshTest(_fixtures.FixtureTest):
         assert u.name == 'jack'
         assert id(a) not in [id(x) for x in u.addresses]
 
-    @testing.resolve_artifact_names
     def test_persistence_check(self):
+        users, User = self.tables.users, self.classes.User
+
         mapper(User, users)
         s = create_session()
         u = s.query(User).get(7)
         s.expunge_all()
         assert_raises_message(sa_exc.InvalidRequestError, r"is not persistent within this Session", lambda: s.refresh(u))
 
-    @testing.resolve_artifact_names
     def test_refresh_expired(self):
+        User, users = self.classes.User, self.tables.users
+
         mapper(User, users)
         s = create_session()
         u = s.query(User).get(7)
@@ -1093,12 +1246,17 @@ class RefreshTest(_fixtures.FixtureTest):
         s.refresh(u)
         assert u.name == 'jack'
 
-    @testing.resolve_artifact_names
     def test_refresh_with_lazy(self):
         """test that when a lazy loader is set as a trigger on an object's attribute
         (at the attribute level, not the class level), a refresh() operation doesnt
         fire the lazy loader or create any problems"""
 
+        User, Address, addresses, users = (self.classes.User,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users)
+
+
         s = create_session()
         mapper(User, users, properties={'addresses':relationship(mapper(Address, addresses))})
         q = s.query(User).options(sa.orm.lazyload('addresses'))
@@ -1107,10 +1265,15 @@ class RefreshTest(_fixtures.FixtureTest):
             s.refresh(u)
         self.assert_sql_count(testing.db, go, 1)
 
-    @testing.resolve_artifact_names
     def test_refresh_with_eager(self):
         """test that a refresh/expire operation loads rows properly and sends correct "isnew" state to eager loaders"""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses), lazy='joined')
         })
@@ -1128,10 +1291,15 @@ class RefreshTest(_fixtures.FixtureTest):
         assert len(u.addresses) == 3
 
     @testing.fails_on('maxdb', 'FIXME: unknown')
-    @testing.resolve_artifact_names
     def test_refresh2(self):
         """test a hang condition that was occurring on expire/refresh"""
 
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
+
         s = create_session()
         mapper(Address, addresses)
 
