@@ -1,26 +1,31 @@
 """basic tests of lazy loaded attributes"""
 
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from test.lib.testing import assert_raises, assert_raises_message
 import datetime
 from sqlalchemy import exc as sa_exc
 from sqlalchemy.orm import attributes, exc as orm_exc
 import sqlalchemy as sa
-from sqlalchemy.test import testing
+from test.lib import testing
 from sqlalchemy import Integer, String, ForeignKey, SmallInteger
 from sqlalchemy.types import TypeDecorator
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from test.lib.schema import Table
+from test.lib.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session
-from sqlalchemy.test.testing import eq_
-from test.orm import _base, _fixtures
+from test.lib.testing import eq_
+from test.lib import fixtures
+from test.orm import _fixtures
 
 
 class LazyTest(_fixtures.FixtureTest):
     run_inserts = 'once'
     run_deletes = None
 
-    @testing.resolve_artifact_names
     def test_basic(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses), lazy='select')
         })
@@ -28,10 +33,15 @@ class LazyTest(_fixtures.FixtureTest):
         q = sess.query(User)
         assert [User(id=7, addresses=[Address(id=1, email_address='jack@bean.com')])] == q.filter(users.c.id == 7).all()
 
-    @testing.resolve_artifact_names
     def test_needs_parent(self):
         """test the error raised when parent object is not bound."""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={
             'addresses':relationship(mapper(Address, addresses), lazy='select')
         })
@@ -41,8 +51,12 @@ class LazyTest(_fixtures.FixtureTest):
         sess.expunge(u)
         assert_raises(orm_exc.DetachedInstanceError, getattr, u, 'addresses')
 
-    @testing.resolve_artifact_names
     def test_orderby(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties = {
             'addresses':relationship(mapper(Address, addresses), lazy='select', order_by=addresses.c.email_address),
         })
@@ -62,10 +76,15 @@ class LazyTest(_fixtures.FixtureTest):
             User(id=10, addresses=[])
         ] == q.all()
 
-    @testing.resolve_artifact_names
     def test_orderby_secondary(self):
         """tests that a regular mapper select on a single table can order by a relationship to a second table"""
 
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
+
         mapper(Address, addresses)
 
         mapper(User, users, properties = dict(
@@ -87,8 +106,12 @@ class LazyTest(_fixtures.FixtureTest):
             ]),
         ] == l
 
-    @testing.resolve_artifact_names
     def test_orderby_desc(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         mapper(Address, addresses)
 
         mapper(User, users, properties = dict(
@@ -110,10 +133,15 @@ class LazyTest(_fixtures.FixtureTest):
             User(id=10, addresses=[])
         ] == sess.query(User).all()
 
-    @testing.resolve_artifact_names
     def test_no_orphan(self):
         """test that a lazily loaded child object is not marked as an orphan"""
 
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+
         mapper(User, users, properties={
             'addresses':relationship(Address, cascade="all,delete-orphan", lazy='select')
         })
@@ -124,10 +152,20 @@ class LazyTest(_fixtures.FixtureTest):
         assert getattr(User, 'addresses').hasparent(attributes.instance_state(user.addresses[0]), optimistic=True)
         assert not sa.orm.class_mapper(Address)._is_orphan(attributes.instance_state(user.addresses[0]))
 
-    @testing.resolve_artifact_names
     def test_limit(self):
         """test limit operations combined with lazy-load relationships."""
 
+        users, items, order_items, orders, Item, User, Address, Order, addresses = (self.tables.users,
+                                self.tables.items,
+                                self.tables.order_items,
+                                self.tables.orders,
+                                self.classes.Item,
+                                self.classes.User,
+                                self.classes.Address,
+                                self.classes.Order,
+                                self.tables.addresses)
+
+
         mapper(Item, items)
         mapper(Order, orders, properties={
             'items':relationship(Item, secondary=order_items, lazy='select')
@@ -147,8 +185,17 @@ class LazyTest(_fixtures.FixtureTest):
             l = q.limit(2).offset(1).all()
             assert self.static.user_all_result[1:3] == l
 
-    @testing.resolve_artifact_names
     def test_distinct(self):
+        users, items, order_items, orders, Item, User, Address, Order, addresses = (self.tables.users,
+                                self.tables.items,
+                                self.tables.order_items,
+                                self.tables.orders,
+                                self.classes.Item,
+                                self.classes.User,
+                                self.classes.Address,
+                                self.classes.Order,
+                                self.tables.addresses)
+
         mapper(Item, items)
         mapper(Order, orders, properties={
             'items':relationship(Item, secondary=order_items, lazy='select')
@@ -167,10 +214,15 @@ class LazyTest(_fixtures.FixtureTest):
         l = q.filter(s.c.u2_id==User.id).order_by(User.id).distinct().all()
         eq_(self.static.user_all_result, l)
 
-    @testing.resolve_artifact_names 
     def test_uselist_false_warning(self):
         """test that multiple rows received by a uselist=False raises a warning."""
 
+        User, users, orders, Order = (self.classes.User,
+                                self.tables.users,
+                                self.tables.orders,
+                                self.classes.Order)
+
+
         mapper(User, users, properties={
             'order':relationship(Order, uselist=False)
         })
@@ -179,8 +231,12 @@ class LazyTest(_fixtures.FixtureTest):
         u1 = s.query(User).filter(User.id==7).one()
         assert_raises(sa.exc.SAWarning, getattr, u1, 'order')
 
-    @testing.resolve_artifact_names
     def test_one_to_many_scalar(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         mapper(User, users, properties = dict(
             address = relationship(mapper(Address, addresses), lazy='select', uselist=False)
         ))
@@ -188,8 +244,12 @@ class LazyTest(_fixtures.FixtureTest):
         l = q.filter(users.c.id == 7).all()
         assert [User(id=7, address=Address(id=1))] == l
 
-    @testing.resolve_artifact_names
     def test_many_to_one_binds(self):
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
         mapper(Address, addresses, primary_key=[addresses.c.user_id, addresses.c.email_address])
 
         mapper(User, users, properties = dict(
@@ -204,15 +264,22 @@ class LazyTest(_fixtures.FixtureTest):
                 User(id=8, address=Address(id=3)),
                 User(id=9, address=None),
                 User(id=10, address=None),
-            ], 
+            ],
             list(q)
         )
 
 
-    @testing.resolve_artifact_names
     def test_double(self):
         """tests lazy loading with two relationships simulatneously, from the same table, using aliases.  """
 
+        users, orders, User, Address, Order, addresses = (self.tables.users,
+                                self.tables.orders,
+                                self.classes.User,
+                                self.classes.Address,
+                                self.classes.Order,
+                                self.tables.addresses)
+
+
         openorders = sa.alias(orders, 'openorders')
         closedorders = sa.alias(orders, 'closedorders')
 
@@ -257,8 +324,13 @@ class LazyTest(_fixtures.FixtureTest):
         assert [Order(id=1), Order(id=5)] == create_session().query(closed_mapper).with_parent(user, property='closed_orders').all()
         assert [Order(id=3)] == create_session().query(open_mapper).with_parent(user, property='open_orders').all()
 
-    @testing.resolve_artifact_names
     def test_many_to_many(self):
+        keywords, items, item_keywords, Keyword, Item = (self.tables.keywords,
+                                self.tables.items,
+                                self.tables.item_keywords,
+                                self.classes.Keyword,
+                                self.classes.Item)
+
 
         mapper(Keyword, keywords)
         mapper(Item, items, properties = dict(
@@ -270,10 +342,15 @@ class LazyTest(_fixtures.FixtureTest):
 
         assert self.static.item_keyword_result[0:2] == q.join('keywords').filter(keywords.c.name == 'red').all()
 
-    @testing.resolve_artifact_names
     def test_uses_get(self):
         """test that a simple many-to-one lazyload optimizes to use query.get()."""
 
+        Address, addresses, users, User = (self.classes.Address,
+                                self.tables.addresses,
+                                self.tables.users,
+                                self.classes.User)
+
+
         for pj in (
             None,
             users.c.id==addresses.c.user_id,
@@ -297,10 +374,12 @@ class LazyTest(_fixtures.FixtureTest):
             self.assert_sql_count(testing.db, go, 0)
             sa.orm.clear_mappers()
 
-    @testing.resolve_artifact_names
     def test_uses_get_compatible_types(self):
         """test the use_get optimization with compatible but non-identical types"""
 
+        User, Address = self.classes.User, self.classes.Address
+
+
         class IntDecorator(TypeDecorator):
             impl = Integer
 
@@ -318,7 +397,7 @@ class LazyTest(_fixtures.FixtureTest):
             SomeDBInteger,
         ]:
             m = sa.MetaData()
-            users = Table('users', m, 
+            users = Table('users', m,
                 Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                 Column('name', String(30), nullable=False),
             )
@@ -346,8 +425,12 @@ class LazyTest(_fixtures.FixtureTest):
             self.assert_sql_count(testing.db, go, 0)
             sa.orm.clear_mappers()
 
-    @testing.resolve_artifact_names
     def test_many_to_one(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(Address, addresses, properties = dict(
             user = relationship(mapper(User, users), lazy='select')
         ))
@@ -361,8 +444,14 @@ class LazyTest(_fixtures.FixtureTest):
 
         assert a.user is u1
 
-    @testing.resolve_artifact_names
+
+
     def test_backrefs_dont_lazyload(self):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users, properties={
             'addresses':relationship(Address, backref='user')
         })
@@ -396,14 +485,137 @@ class LazyTest(_fixtures.FixtureTest):
             assert ad2 in u1.addresses
         self.assert_sql_count(testing.db, go, 1)
 
+class GetterStateTest(_fixtures.FixtureTest):
+    """test lazyloader on non-existent attribute returns
+    expected attribute symbols, maintain expected state"""
+
+    run_inserts = None
+
+    def _u_ad_fixture(self, populate_user):
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
+        mapper(User, users, properties={
+            'addresses':relationship(Address, backref='user')
+        })
+        mapper(Address, addresses)
+
+        sess = create_session()
+        a1 = Address(email_address='a1')
+        sess.add(a1)
+        if populate_user:
+            a1.user = User(name='ed')
+        sess.flush()
+        if populate_user:
+            sess.expire_all()
+        return User, Address, sess, a1
+
+    def test_get_empty_passive_return_never_set(self):
+        User, Address, sess, a1 = self._u_ad_fixture(False)
+        eq_(
+            Address.user.impl.get(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_RETURN_NEVER_SET),
+            attributes.NEVER_SET
+        )
+        assert 'user_id' not in a1.__dict__
+        assert 'user' not in a1.__dict__
+
+    def test_history_empty_passive_return_never_set(self):
+        User, Address, sess, a1 = self._u_ad_fixture(False)
+        eq_(
+            Address.user.impl.get_history(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_RETURN_NEVER_SET),
+            ((), (), ())
+        )
+        assert 'user_id' not in a1.__dict__
+        assert 'user' not in a1.__dict__
+
+    def test_get_empty_passive_no_initialize(self):
+        User, Address, sess, a1 = self._u_ad_fixture(False)
+        eq_(
+            Address.user.impl.get(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_NO_INITIALIZE),
+            attributes.PASSIVE_NO_RESULT
+        )
+        assert 'user_id' not in a1.__dict__
+        assert 'user' not in a1.__dict__
+
+    def test_history_empty_passive_no_initialize(self):
+        User, Address, sess, a1 = self._u_ad_fixture(False)
+        eq_(
+            Address.user.impl.get_history(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_NO_INITIALIZE),
+            attributes.HISTORY_BLANK
+        )
+        assert 'user_id' not in a1.__dict__
+        assert 'user' not in a1.__dict__
+
+    def test_get_populated_passive_no_initialize(self):
+        User, Address, sess, a1 = self._u_ad_fixture(True)
+        eq_(
+            Address.user.impl.get(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_NO_INITIALIZE),
+            attributes.PASSIVE_NO_RESULT
+        )
+        assert 'user_id' not in a1.__dict__
+        assert 'user' not in a1.__dict__
+
+    def test_history_populated_passive_no_initialize(self):
+        User, Address, sess, a1 = self._u_ad_fixture(True)
+        eq_(
+            Address.user.impl.get_history(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_NO_INITIALIZE),
+            attributes.HISTORY_BLANK
+        )
+        assert 'user_id' not in a1.__dict__
+        assert 'user' not in a1.__dict__
+
+    def test_get_populated_passive_return_never_set(self):
+        User, Address, sess, a1 = self._u_ad_fixture(True)
+        eq_(
+            Address.user.impl.get(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_RETURN_NEVER_SET),
+            User(name='ed')
+        )
+
+    def test_history_populated_passive_return_never_set(self):
+        User, Address, sess, a1 = self._u_ad_fixture(True)
+        eq_(
+            Address.user.impl.get_history(
+                attributes.instance_state(a1),
+                attributes.instance_dict(a1),
+                passive=attributes.PASSIVE_RETURN_NEVER_SET),
+            ((), [User(name='ed'), ], ())
+        )
 
 class M2OGetTest(_fixtures.FixtureTest):
     run_inserts = 'once'
     run_deletes = None
 
-    @testing.resolve_artifact_names
     def test_m2o_noload(self):
         """test that a NULL foreign key doesn't trigger a lazy load"""
+
+        users, Address, addresses, User = (self.tables.users,
+                                self.classes.Address,
+                                self.tables.addresses,
+                                self.classes.User)
+
         mapper(User, users)
 
         mapper(Address, addresses, properties={
@@ -425,7 +637,7 @@ class M2OGetTest(_fixtures.FixtureTest):
             assert ad3.user is None
         self.assert_sql_count(testing.db, go, 1)
 
-class CorrelatedTest(_base.MappedTest):
+class CorrelatedTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(self, meta):
@@ -439,8 +651,9 @@ class CorrelatedTest(_base.MappedTest):
               Column('user_id', Integer, ForeignKey('user_t.id')))
 
     @classmethod
-    @testing.resolve_artifact_names
     def insert_data(cls):
+        stuff, user_t = cls.tables.stuff, cls.tables.user_t
+
         user_t.insert().execute(
             {'id':1, 'name':'user1'},
             {'id':2, 'name':'user2'},
@@ -453,12 +666,13 @@ class CorrelatedTest(_base.MappedTest):
             {'id':4, 'user_id':2, 'date':datetime.date(2008, 1, 15)},
             {'id':5, 'user_id':3, 'date':datetime.date(2007, 6, 15)})
 
-    @testing.resolve_artifact_names
     def test_correlated_lazyload(self):
-        class User(_base.ComparableEntity):
+        stuff, user_t = self.tables.stuff, self.tables.user_t
+
+        class User(fixtures.ComparableEntity):
             pass
 
-        class Stuff(_base.ComparableEntity):
+        class Stuff(fixtures.ComparableEntity):
             pass
 
         mapper(Stuff, stuff)
