@@ -1,16 +1,16 @@
-from sqlalchemy.test.testing import assert_raises, \
+from test.lib.testing import assert_raises, \
     assert_raises_message, eq_
 import sqlalchemy as sa
-from sqlalchemy.test import testing
+from test.lib import testing
 from sqlalchemy import Integer, String, ForeignKey
-from sqlalchemy.test.schema import Table
-from sqlalchemy.test.schema import Column
+from test.lib.schema import Table
+from test.lib.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
     exc as orm_exc, sessionmaker
-from test.orm import _base
+from test.lib import fixtures
 
 
-class M2MTest(_base.MappedTest):
+class M2MTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('place', metadata,
@@ -60,18 +60,18 @@ class M2MTest(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Place(_base.BasicEntity):
+        class Place(cls.Basic):
             def __init__(self, name=None):
                 self.name = name
             def __str__(self):
                 return "(Place '%s')" % self.name
             __repr__ = __str__
 
-        class PlaceThingy(_base.BasicEntity):
+        class PlaceThingy(cls.Basic):
             def __init__(self, name=None):
                 self.name = name
 
-        class Transition(_base.BasicEntity):
+        class Transition(cls.Basic):
             def __init__(self, name=None):
                 self.name = name
                 self.inputs = []
@@ -81,8 +81,13 @@ class M2MTest(_base.MappedTest):
                                  repr(self.inputs),
                                  repr(self.outputs)))
 
-    @testing.resolve_artifact_names
     def test_error(self):
+        place, Transition, place_input, Place, transition = (self.tables.place,
+                                self.classes.Transition,
+                                self.tables.place_input,
+                                self.classes.Place,
+                                self.tables.transition)
+
         mapper(Place, place, properties={
             'transitions':relationship(Transition, secondary=place_input, backref='places')
         })
@@ -90,16 +95,19 @@ class M2MTest(_base.MappedTest):
             'places':relationship(Place, secondary=place_input, backref='transitions')
         })
         assert_raises_message(sa.exc.ArgumentError, "Error creating backref",
-                                 sa.orm.compile_mappers)
+                                 sa.orm.configure_mappers)
 
-    @testing.resolve_artifact_names
     def test_circular(self):
         """test a many-to-many relationship from a table to itself."""
 
+        place, Place, place_place = (self.tables.place,
+                                self.classes.Place,
+                                self.tables.place_place)
+
         mapper(Place, place, properties={
             'places': relationship(
                         Place,
-                        secondary=place_place, 
+                        secondary=place_place,
                         primaryjoin=place.c.place_id==place_place.c.pl1_id,
                         secondaryjoin=place.c.place_id==place_place.c.pl2_id,
                         order_by=place_place.c.pl2_id
@@ -143,14 +151,18 @@ class M2MTest(_base.MappedTest):
         [sess.delete(p) for p in p1,p2,p3,p4,p5,p6,p7]
         sess.flush()
 
-    @testing.resolve_artifact_names
     def test_circular_mutation(self):
         """Test that a mutation in a self-ref m2m of both sides succeeds."""
 
+
+        place, Place, place_place = (self.tables.place,
+                                self.classes.Place,
+                                self.tables.place_place)
+
         mapper(Place, place, properties={
             'child_places': relationship(
                         Place,
-                        secondary=place_place, 
+                        secondary=place_place,
                         primaryjoin=place.c.place_id==place_place.c.pl1_id,
                         secondaryjoin=place.c.place_id==place_place.c.pl2_id,
                         order_by=place_place.c.pl2_id,
@@ -171,11 +183,20 @@ class M2MTest(_base.MappedTest):
         assert p2 in p1.parent_places
 
 
-    @testing.resolve_artifact_names
     def test_double(self):
         """test that a mapper can have two eager relationships to the same table, via
         two different association tables.  aliases are required."""
 
+        place_input, transition, Transition, PlaceThingy, place, place_thingy, Place, place_output = (self.tables.place_input,
+                                self.tables.transition,
+                                self.classes.Transition,
+                                self.classes.PlaceThingy,
+                                self.tables.place,
+                                self.tables.place_thingy,
+                                self.classes.Place,
+                                self.tables.place_output)
+
+
         Place.mapper = mapper(Place, place, properties = {
             'thingies':relationship(mapper(PlaceThingy, place_thingy), lazy='joined')
         })
@@ -202,9 +223,14 @@ class M2MTest(_base.MappedTest):
             'outputs': (Place, [{'name':'place2'}, {'name':'place3'}])
             })
 
-    @testing.resolve_artifact_names
     def test_bidirectional(self):
-        """tests a many-to-many backrefs"""
+        place_input, transition, Transition, Place, place, place_output = (self.tables.place_input,
+                                self.tables.transition,
+                                self.classes.Transition,
+                                self.classes.Place,
+                                self.tables.place,
+                                self.tables.place_output)
+
         Place.mapper = mapper(Place, place)
         Transition.mapper = mapper(Transition, transition, properties = dict(
             inputs = relationship(Place.mapper, place_output, lazy='select', backref='inputs'),
@@ -234,10 +260,15 @@ class M2MTest(_base.MappedTest):
         self.assert_result([p2], Place, {'inputs': (Transition, [{'name':'transition1'},{'name':'transition2'}])})
 
     @testing.requires.sane_multi_rowcount
-    @testing.resolve_artifact_names
     def test_stale_conditions(self):
+        Place, Transition, place_input, place, transition = (self.classes.Place,
+                                self.classes.Transition,
+                                self.tables.place_input,
+                                self.tables.place,
+                                self.tables.transition)
+
         mapper(Place, place, properties={
-            'transitions':relationship(Transition, secondary=place_input, 
+            'transitions':relationship(Transition, secondary=place_input,
                                             passive_updates=False)
         })
         mapper(Transition, transition)
@@ -274,7 +305,7 @@ class M2MTest(_base.MappedTest):
             sess.commit
         )
 
-class M2MTest2(_base.MappedTest):
+class M2MTest2(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('student', metadata,
@@ -291,15 +322,20 @@ class M2MTest2(_base.MappedTest):
 
     @classmethod
     def setup_classes(cls):
-        class Student(_base.BasicEntity):
+        class Student(cls.Basic):
             def __init__(self, name=''):
                 self.name = name
-        class Course(_base.BasicEntity):
+        class Course(cls.Basic):
             def __init__(self, name=''):
                 self.name = name
 
-    @testing.resolve_artifact_names
     def test_circular(self):
+        course, enroll, Student, student, Course = (self.tables.course,
+                                self.tables.enroll,
+                                self.classes.Student,
+                                self.tables.student,
+                                self.classes.Course)
+
 
         mapper(Student, student)
         mapper(Course, course, properties={
@@ -324,10 +360,16 @@ class M2MTest2(_base.MappedTest):
         del s.courses[1]
         self.assert_(len(s.courses) == 2)
 
-    @testing.resolve_artifact_names
     def test_dupliates_raise(self):
         """test constraint error is raised for dupe entries in a list"""
 
+        course, enroll, Student, student, Course = (self.tables.course,
+                                self.tables.enroll,
+                                self.classes.Student,
+                                self.tables.student,
+                                self.classes.Course)
+
+
         mapper(Student, student)
         mapper(Course, course, properties={
             'students': relationship(Student, enroll, backref='courses')})
@@ -340,10 +382,16 @@ class M2MTest2(_base.MappedTest):
         sess.add(s1)
         assert_raises(sa.exc.DBAPIError, sess.flush)
 
-    @testing.resolve_artifact_names
     def test_delete(self):
         """A many-to-many table gets cleared out with deletion from the backref side"""
 
+        course, enroll, Student, student, Course = (self.tables.course,
+                                self.tables.enroll,
+                                self.classes.Student,
+                                self.tables.student,
+                                self.classes.Course)
+
+
         mapper(Student, student)
         mapper(Course, course, properties = {
             'students': relationship(Student, enroll, lazy='select',
@@ -363,7 +411,7 @@ class M2MTest2(_base.MappedTest):
         sess.flush()
         assert enroll.count().scalar() == 0
 
-class M2MTest3(_base.MappedTest):
+class M2MTest3(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('c', metadata,
@@ -388,8 +436,13 @@ class M2MTest3(_base.MappedTest):
             Column('a1', Integer, ForeignKey('a.a1')),
             Column('b2', sa.Boolean))
 
-    @testing.resolve_artifact_names
     def test_basic(self):
+        a, c, b, c2a1, c2a2 = (self.tables.a,
+                                self.tables.c,
+                                self.tables.b,
+                                self.tables.c2a1,
+                                self.tables.c2a2)
+
         class C(object):pass
         class A(object):pass
         class B(object):pass
@@ -410,7 +463,7 @@ class M2MTest3(_base.MappedTest):
         # TODO: seems like just a test for an ancient exception throw.
         # how about some data/inserts/queries/assertions for this one
 
-class M2MTest4(_base.MappedTest):
+class M2MTest4(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         table1 = Table("table1", metadata,
@@ -428,11 +481,14 @@ class M2MTest4(_base.MappedTest):
             Column('t2', Integer, ForeignKey('table2.col1')),
             )
 
-    @testing.resolve_artifact_names
     def test_delete_parent(self):
-        class A(_base.ComparableEntity):
+        table2, table3, table1 = (self.tables.table2,
+                                self.tables.table3,
+                                self.tables.table1)
+
+        class A(fixtures.ComparableEntity):
             pass
-        class B(_base.ComparableEntity):
+        class B(fixtures.ComparableEntity):
             pass
 
         mapper(A, table1, properties={
